/*******************************************************************************
 * Copyright Moxa Inc.
 *
 *
 * Date          Author            Comment
 * 02/25/2019    Daniel Chang      Created.
 ******************************************************************************/

 /*!
    \file Integrate2dll.cpp
    \copyright Moxa Inc.
    \brief <b>\a Integrate2dll \a MXIO_V3 \a and \a MXIO </b>
    \date 02/25/2019
    \author Daniel Chang
    \version V1.0
    \details
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <MXIO_V3_2500.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>


#include <mxio.h>
#include <MXIO_V3_4500.h>
//-----------------------------------------------------------------
/*#if defined(_WIN64)
	#pragma comment(lib, "AMD64/MXIO_VC.lib")       //64bit MXIO_VC.lib
#else
	#pragma comment(lib, "vc/MXIO_VC.lib")			//32bit MXIO_VC.lib
#endif*/

#define HMODULE void
//-----------------------------------------------------------------
#define TRUE    1
#define FALSE   0

#define HMODULE void
//-----------------------------------------------------------------
//Common Function
char getch();
void WaitPressAnyKey();
void CheckErr(int ihConnection, int iRet, char *szFunctionName);
void PrtStrErrorString(WORD wErrorCode, char *szName);
void PrtStrChErrorString(BYTE bytErrorCode, char* pszName);

//-----------------------------------------------------------------
//ioLogik 2500 Series
int MXCreateLibrary(HMODULE **phMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500);
void PrintReadValue(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintCounterActiveModeStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintTemperatureUnitStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintCHInfoStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp);
void CALLBACK ClearDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvClearBlockData, void *pParam);
void CALLBACK WriteDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvWriteBlockData, void *pParam);
void CALLBACK ReadDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvReadBlockData, void *pParam);
//
void *Thread_2500( void * lParam );
//
HMODULE *g_hMXIO_V3_2500 = NULL;
CMXIO_V3_2500 *g_pMXIO_V3_2500 = NULL;


//-----------------------------------------------------------------
//ioLogik 4500 Series
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500);
void PrintReadValue(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintCounterActiveModeStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintTemperatureUnitStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp);
BOOL PrintCHInfoStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp);
void CALLBACK ClearDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvClearBlockData, void *pParam);
void CALLBACK WriteDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvWriteBlockData, void *pParam);
void CALLBACK ReadDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvReadBlockData, void *pParam);
//
void *Thread_4500( void * lParam );
//
HMODULE *g_hMXIO_V3_4500 = NULL;
CMXIO_V3_4500 *g_pMXIO_V3_4500 = NULL;

//-----------------------------------------------------------------
#pragma pack(1)

// For thread used
typedef struct _DEVICE_PARAMETERS{
	char IPAddress[16];
	WORD wPort;
	DWORD dwTimeout;
	char *szMACAddr;
	char *szUserName;
	char *szPwd;
	bool bStop;
}DEVICE_PARAMETERS;

//-----------------------------------------------------------------
#define BLOCK_SIZE  1

//-----------------------------------------------------------------
//! \include Integrate2dll.cpp
int main(int argc, char *argv[])
{
	char bytInputValue[64] = {'\0'};
    CMXIO_V3_2500::MX_ACCOUNT_DATA AccountData;
	sprintf(AccountData.szUserName,"%s", (char*)"admin");
	sprintf(AccountData.szPwd,"%s", (char*)"moxa");

    //-----------------------------------------------
    // For 2500 default IP Address
    char IPAddress[16] = "192.168.127.253";
    //-----------------------------------------------
    // For 4500 default IP Address
    char IPAddress2[16] = "192.168.127.250";

    WORD wPort = MXIO_V3_AGENT_PORT;
    DWORD dwTimeout = 3000;
	const char* pszMACAddress = "00-00-00-00-00-00"; //"00-90-E8-00-00-41";

    if (argc <= 1)
    {
        printf("Cmd Format: [IP Address] [Timeout] [TCP Port] [IP Address2]\n");
        printf("Press Enter to exit\n");
        getchar();
        return 0;
    }
    //
    if (argc > 1)
        sprintf(IPAddress, "%s", argv[1]);
    if (argc > 2)
        dwTimeout = atoi(argv[2]);
    if (argc > 3)
        wPort = atoi(argv[3]);
    if (argc > 4)
		sprintf(IPAddress2, "%s", argv[4]);
	printf("Input IP Address=%s, Timeout=%d, TCP Port=%d, IP Address2=%s\n",
		IPAddress, dwTimeout, wPort, IPAddress2);
	//-----------------------
	#define THREAD_QUANTITY		2

	HANDLE hThread[THREAD_QUANTITY];
	#ifdef WINDOWS_TYPE
        DWORD dwthread[THREAD_QUANTITY];
    #else
        pthread_t dwthread[THREAD_QUANTITY];
    #endif

	void *thread_result[THREAD_QUANTITY];
    int iDevice = (sizeof(dwthread)/sizeof(dwthread[0]));
    DEVICE_PARAMETERS *DeviceParameters = new DEVICE_PARAMETERS[iDevice];
	printf("DeviceParameters=%p\r\n", DeviceParameters);

	//Initial
	for(int i=0; i < iDevice; i++)
	{
		hThread[i] = 0;
		DeviceParameters[i].wPort = wPort;
		DeviceParameters[i].dwTimeout = dwTimeout;
		DeviceParameters[i].szMACAddr = (char*)pszMACAddress;
		DeviceParameters[i].szUserName = AccountData.szUserName;
		DeviceParameters[i].szPwd = AccountData.szPwd;
		DeviceParameters[i].bStop = FALSE;
	}

	//2500 Lib File handle
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500, (char*)"/usr/local/lib/libmxio_2500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500, (char*)"/usr/local/lib/libmxio_2500.so");
#endif
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
        //Thread [0] for 2500
		strcpy(DeviceParameters[0].IPAddress, IPAddress);
		hThread[0] = pthread_create( &dwthread[0], 0, Thread_2500, (void *)&DeviceParameters[0]);
	}

	//4500 Lib File handle
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500.so");
#endif
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
	    //Thread [1] for 4500
		strcpy(DeviceParameters[1].IPAddress, IPAddress);
		hThread[1] = pthread_create( &dwthread[1], 0, Thread_4500, (void *)&DeviceParameters[1]);
	}

	if (MXIO_V3_OK == iRet)
	{
	    //allow the thread to do some work:
		for (int i = 10; i > 0; i--)
		{
		    sleep(3);
		}
	}
	//Stop thread
	for (int i = 0; i < iDevice; i++)
		DeviceParameters[i].bStop = TRUE;

    //wait thread finish
	for (int i = 0; i < iDevice; i++)
	{
	    iRet = pthread_join(dwthread[i], NULL);
	}

	//Free memory
	delete [] DeviceParameters;

    if (NULL != g_hMXIO_V3_2500)
    {
        iRet = g_pMXIO_V3_2500->MXIO_Exit();
        iRet = MXDestroyLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500);
    }
    //
	if (NULL != g_hMXIO_V3_4500)
    {
        iRet = g_pMXIO_V3_4500->MXIO_Exit();
        iRet = MXDestroyLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500);
    }
	
	WaitPressAnyKey();
	return 0;
}
//-----------------------------------------------------------------
// Common Function
//-----------------------------------------------------------------
char getch()
{
    int iRet;
	char ch;
	struct termios oldt, newt;

	tcgetattr(STDIN_FILENO, &oldt);
	memcpy(&newt, &oldt, sizeof(newt));
	newt.c_lflag &= ~( ECHO | ICANON | ECHOE | ECHOK |
						ECHONL | ECHOPRT | ECHOKE | ICRNL);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	iRet = read(STDIN_FILENO, &ch, 1);
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if(iRet == 1)
		return ch;
	else
		return 0;
}

void WaitPressAnyKey()
{
	printf("press any key to exit.\n");
	getch();
}

//-----------------------------------------------------------------
//===========================================================================
void CheckErr(int ihConnection, int iRet, char *szFunctionName)
{
	char *szErrMsg = (char*)"MXIO_V3_OK";
	if (iRet != MXIO_V3_OK)
	{
		switch (iRet)
		{
		case MXIO_V3_ILLEGAL_FUNCTION:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_FUNCTION";
			break;
		case MXIO_V3_ILLEGAL_DATA_ADDRESS:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_ADDRESS";
			break;
		case MXIO_V3_ILLEGAL_DATA_VALUE:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_VALUE";
			break;
		case MXIO_V3_SLAVE_DEVICE_FAILURE:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_FAILURE";
			break;
		case MXIO_V3_SLAVE_DEVICE_BUSY:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_BUSY";
			break;
		case MXIO_V3_EIO_TIME_OUT:
			szErrMsg = (char*)"MXIO_V3_EIO_TIME_OUT";
			break;
		case MXIO_V3_EIO_INIT_SOCKETS_FAIL:
			szErrMsg = (char*)"MXIO_V3_EIO_INIT_SOCKETS_FAIL";
			break;
		case MXIO_V3_EIO_CREATING_SOCKET_ERROR:
			szErrMsg = (char*)"MXIO_V3_EIO_CREATING_SOCKET_ERROR";
			break;
		case MXIO_V3_EIO_RESPONSE_BAD:
			szErrMsg = (char*)"MXIO_V3_EIO_RESPONSE_BAD";
			break;
		case MXIO_V3_EIO_SOCKET_DISCONNECT:
			szErrMsg = (char*)"MXIO_V3_EIO_SOCKET_DISCONNECT";
			break;
		case MXIO_V3_PROTOCOL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_PROTOCOL_TYPE_ERROR";
			break;
		case MXIO_V3_PRODUCT_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_PRODUCT_NOT_SUPPORT";
			break;
		case MXIO_V3_HANDLE_ERROR:
		    szErrMsg = (char*)"MXIO_V3_HANDLE_ERROR";
		    break;
		case MXIO_V3_SLOT_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_SLOT_OUT_OF_RANGE";
			break;
		case MXIO_V3_CHANNEL_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_CHANNEL_OUT_OF_RANGE";
			break;
		case MXIO_V3_COIL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_COIL_TYPE_ERROR";
			break;
		case MXIO_V3_REGISTER_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_REGISTER_TYPE_ERROR";
			break;
		case MXIO_V3_FUNCTION_NOT_SUPPORT:
		    szErrMsg = (char*)"MXIO_V3_FUNCTION_NOT_SUPPORT";
		    break;
		case MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_INPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_INPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_EIO_PASSWORD_INCORRECT:
			szErrMsg = (char*)"MXIO_V3_EIO_PASSWORD_INCORRECT";
			break;
		case MXIO_V3_SLOT_NOT_EXIST:
			szErrMsg = (char*)"MXIO_V3_SLOT_NOT_EXIST";
			break;
		case MXIO_V3_FIRMWARE_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_FIRMWARE_NOT_SUPPORT";
			break;
		case MXIO_V3_CREATE_MUTEX_FAIL:
			szErrMsg = (char*)"MXIO_V3_CREATE_MUTEX_FAIL";
			break;
		case MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_FUNCTION_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_ERR_FUNCTION_NOT_SUPPORT";
			break;
		case MXIO_V3_ERR_INVALID_HANDLE:
			szErrMsg = (char*)"MXIO_V3_ERR_INVALID_HANDLE";
			break;
		case MXIO_V3_ERR_BLOCK_QUANTITY:
			szErrMsg = (char*)"MXIO_V3_ERR_BLOCK_QUANTITY";
			break;
		case MXIO_V3_ERR_TOTAL_BLOCK_SIZE:
			szErrMsg = (char*)"MXIO_V3_ERR_TOTAL_BLOCK_SIZE";
			break;
		case MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION:
			szErrMsg = (char*)"MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION";
			break;
		case MXIO_V3_ERR_PARSE_XML_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_PARSE_XML_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT:
			szErrMsg = (char *)"MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT";
			break;
		case MXIO_V3_PARSING_DEVICE_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_DEVICE_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_CHANNEL_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_CHANNEL_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_INPUT_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_INPUT_NAME_ERROR";
			break;
		default:
			szErrMsg = (char*)"Unknown Error Value";
			break;
		}
		printf("%d Function \"%s\" execution Fail.\nError Message : %s (%d(0x%X))\n",
			   ihConnection, szFunctionName, szErrMsg, iRet, iRet);
	}
	else
	{
		printf("%d Function \"%s\" execution Success\n", ihConnection, szFunctionName);
	}
}

//-----------------------------------------------------------------
void PrtStrErrorString(WORD wErrorCode, char *szName)
{
	if (MXIO_V3_BLOCK_ERR_SIZE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SIZE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_TYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SUBTYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SUBTYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_FUNCTION == wErrorCode)
		printf("[%s] = BLOCK_ERR_FUNCTION\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SLOT == wErrorCode)
		printf("[%s] = BLOCK_ERR_SLOT\n", szName);
	else if (MXIO_V3_BLOCK_ERR_START_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_START_CHANNEL\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TOTAL_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_TOTAL_CHANNEL\n", szName);
	else
		printf("[%s] = Not defined error code= %d\n", szName, wErrorCode);
}
//-----------------------------------------------------------------
void PrtChErrorString(int iIndex, BYTE bytErrorCode)
{
	if (MXIO_V3_CH_OK == bytErrorCode)
		printf("CH[%d] = SUCCESS\n", iIndex);
	else if (MXIO_V3_CH_ERR_LOCKED == bytErrorCode)
		printf("CH[%d] = ERROR_LOCKED\n", iIndex);
	else if (MXIO_V3_CH_ERR_READ_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_READ_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_SUBTYPE == bytErrorCode)
		printf("CH[%d] = ERROR_SUBTYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA_TYPE == bytErrorCode)
		printf("CH[%d] = ERROR_DATA_TYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA == bytErrorCode)
		printf("CH[%d] = ERROR_DATA\n", iIndex);
	else if (MXIO_V3_CH_ERR_DISABLE == bytErrorCode)
		printf("CH[%d] = ERROR_DISABLE\n", iIndex);
	else if (MXIO_V3_CH_ERR_BOOLEAN_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_BOOLEAN_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_WRITE_FAIL== bytErrorCode)
		printf("CH[%d] = MXIO_V3_CH_ERR_WRITE_FAIL\n", iIndex);
	else
		printf("CH[%d] = Not defined error code= %d\n", iIndex, bytErrorCode);
}

//-----------------------------------------------------------------
//ioLogik 2500 Series
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500, char *szLibName)
{
	phMXIO_V3_2500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_2500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_2500 pfnCreateMXIO_V3_2500 = (PFNCREATEMXIO_V3_2500)dlsym(phMXIO_V3_2500, "CreateMXIO_2500");
	if (NULL == pfnCreateMXIO_V3_2500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_2500 = pfnCreateMXIO_V3_2500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500)
{
	PFNDESTROYMXIO_V3_2500 pfnDestroyMXIO_V3_2500 = (PFNDESTROYMXIO_V3_2500)dlsym(hMXIO_V3_2500, "DestroyMXIO_2500");
	if (!pfnDestroyMXIO_V3_2500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_2500(*pMXIO_V3_2500);
	dlclose(hMXIO_V3_2500);
	*pMXIO_V3_2500 = NULL;
	return 0;
}
//-----------------------------------------------------------------
typedef struct _FUNC_CHINFO{
    WORD wIOType;
    WORD wFunc;
    DWORD dwChInfo;
    char* szChInfo;
}FUNC_CHINFO, *pFUNC_CHINFO;
//
FUNC_CHINFO FuncInfo[] = {
    {IOTYPE_DI,       IOFUNC_DI_CH_MODE,             CHINFO_DI_DI,                      (char*)"CHINFO_DI_DI"},
    {IOTYPE_DI,       IOFUNC_DI_CH_MODE,             CHINFO_DI_COUNTER,                 (char*)"CHINFO_DI_COUNTER"},
    {IOTYPE_DO,       IOFUNC_DO_CH_MODE,             CHINFO_DO_DO,                      (char*)"CHINFO_DO_DO"},
    {IOTYPE_DO,       IOFUNC_DO_CH_MODE,             CHINFO_DO_PULSE,                   (char*)"CHINFO_DO_PULSE"},
    {IOTYPE_DIO,      IOFUNC_DIO_CH_MODE,            CHINFO_DIO_DI,                     (char*)"CHINFO_DIO_DI"},
    {IOTYPE_DIO,      IOFUNC_DIO_CH_MODE,            CHINFO_DIO_COUNTER,                (char*)"CHINFO_DIO_COUNTER"},
    {IOTYPE_DIO,      IOFUNC_DIO_CH_MODE,            CHINFO_DIO_DO,                     (char*)"CHINFO_DIO_DO"},
    {IOTYPE_DIO,      IOFUNC_DIO_CH_MODE,            CHINFO_DIO_PULSE,                  (char*)"CHINFO_DIO_PULSE"},
    {IOTYPE_RELAY,    IOFUNC_RELAY_CH_MODE,          CHINFO_RELAY_RELAY,                (char*)"CHINFO_RELAY_RELAY"},
    {IOTYPE_RELAY,	  IOFUNC_RELAY_CH_MODE,			 CHINFO_RELAY_PULSE,                (char*)"CHINFO_RELAY_PULSE"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_DISABLED,                (char*)"CHINFO_AI_DISABLED"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_150mV,                   (char*)"CHINFO_AI_150mV"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_500mV,                   (char*)"CHINFO_AI_500mV"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_5V,                      (char*)"CHINFO_AI_5V"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_10V,                     (char*)"CHINFO_AI_10V"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_0_20mA,                  (char*)"CHINFO_AI_0_20mA"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_4_20mA,                  (char*)"CHINFO_AI_4_20mA"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_0_10V,                   (char*)"CHINFO_AI_0_10V"},
    {IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_4_20mA_BurnOut,          (char*)"CHINFO_AI_4_20mA_BurnOut"},
    {IOTYPE_AO,          IOFUNC_AO_CH_MODE,          CHINFO_AO_DISABLED,                (char*)"CHINFO_AO_DISABLED"},
    {IOTYPE_AO,          IOFUNC_AO_CH_MODE,          CHINFO_AO_4_20mA,                  (char*)"CHINFO_AO_4_20mA"},
    {IOTYPE_AO,          IOFUNC_AO_CH_MODE,          CHINFO_AO_0_10V,                   (char*)"CHINFO_AO_0_10V"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_DISABLED,                (char*)"CHINFO_TC_DISABLED"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_J_TYPE,                  (char*)"CHINFO_TC_J_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_K_TYPE,                  (char*)"CHINFO_TC_K_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_T_TYPE,                  (char*)"CHINFO_TC_T_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_E_TYPE,                  (char*)"CHINFO_TC_E_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_R_TYPE,                  (char*)"CHINFO_TC_R_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_S_TYPE,                  (char*)"CHINFO_TC_S_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_B_TYPE,                  (char*)"CHINFO_TC_B_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_N_TYPE,                  (char*)"CHINFO_TC_N_TYPE"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_78126,               (char*)"CHINFO_TC_VOL_78126"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_39062,               (char*)"CHINFO_TC_VOL_39062"},
    {IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_19532,               (char*)"CHINFO_TC_VOL_19532"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_DISABLED,               (char*)"CHINFO_RTD_DISABLED"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT100,                  (char*)"CHINFO_RTD_PT100"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT200,                  (char*)"CHINFO_RTD_PT200"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT500,                  (char*)"CHINFO_RTD_PT500"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT1000,                 (char*)"CHINFO_RTD_PT1000"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT100,                 (char*)"CHINFO_RTD_JPT100"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT200,                 (char*)"CHINFO_RTD_JPT200"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT500,                 (char*)"CHINFO_RTD_JPT500"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT1000,                (char*)"CHINFO_RTD_JPT1000"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI100,                  (char*)"CHINFO_RTD_NI100"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI200,                  (char*)"CHINFO_RTD_NI200"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI500,                  (char*)"CHINFO_RTD_NI500"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI1000,                 (char*)"CHINFO_RTD_NI1000"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_310,                (char*)"CHINFO_RTD_RES_310"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_620,                (char*)"CHINFO_RTD_RES_620"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_1250,               (char*)"CHINFO_RTD_RES_1250"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_2200,               (char*)"CHINFO_RTD_RES_2200"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT50,                   (char*)"CHINFO_RTD_PT50"},
    {IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI120,                  (char*)"CHINFO_RTD_NI120"},
    //
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DI,         CHINFO_DI_DI,             (char*)"CHINFO_DI_DI"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DI,         CHINFO_DI_COUNTER,        (char*)"CHINFO_DI_COUNTER"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DO,         CHINFO_DO_DO,             (char*)"CHINFO_DO_DO"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DO,         CHINFO_DO_PULSE,          (char*)"CHINFO_DO_PULSE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DIO,        CHINFO_DIO_DI,            (char*)"CHINFO_DIO_DI"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DIO,        CHINFO_DIO_COUNTER,       (char*)"CHINFO_DIO_COUNTER"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DIO,        CHINFO_DIO_DO,            (char*)"CHINFO_DIO_DO"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_DIO,        CHINFO_DIO_PULSE,         (char*)"CHINFO_DIO_PULSE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RELAY,      CHINFO_RELAY_RELAY,       (char*)"CHINFO_RELAY_RELAY"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RELAY,      CHINFO_RELAY_PULSE,       (char*)"CHINFO_RELAY_PULSE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_DISABLED,       (char*)"CHINFO_AI_DISABLED"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_150mV,          (char*)"CHINFO_AI_150mV"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_500mV,          (char*)"CHINFO_AI_500mV"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_5V,             (char*)"CHINFO_AI_5V"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_10V,            (char*)"CHINFO_AI_10V"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_0_20mA,         (char*)"CHINFO_AI_0_20mA"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_4_20mA,         (char*)"CHINFO_AI_4_20mA"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_0_10V,          (char*)"CHINFO_AI_0_10V"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AI,         CHINFO_AI_4_20mA_BurnOut, (char*)"CHINFO_AI_4_20mA_BurnOut"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AO,         CHINFO_AO_DISABLED,       (char*)"CHINFO_AO_DISABLED"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AO,         CHINFO_AO_4_20mA,         (char*)"CHINFO_AO_4_20mA"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_AO,         CHINFO_AO_0_10V,          (char*)"CHINFO_AO_0_10V"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_DISABLED,       (char*)"CHINFO_TC_DISABLED"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_J_TYPE,         (char*)"CHINFO_TC_J_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_K_TYPE,         (char*)"CHINFO_TC_K_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_T_TYPE,         (char*)"CHINFO_TC_T_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_E_TYPE,         (char*)"CHINFO_TC_E_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_R_TYPE,         (char*)"CHINFO_TC_R_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_S_TYPE,         (char*)"CHINFO_TC_S_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_B_TYPE,         (char*)"CHINFO_TC_B_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_N_TYPE,         (char*)"CHINFO_TC_N_TYPE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_VOL_78126,      (char*)"CHINFO_TC_VOL_78126"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_VOL_39062,      (char*)"CHINFO_TC_VOL_39062"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_TC,         CHINFO_TC_VOL_19532,      (char*)"CHINFO_TC_VOL_19532"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_DISABLED,      (char*)"CHINFO_RTD_DISABLED"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_PT100,         (char*)"CHINFO_RTD_PT100"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_PT200,         (char*)"CHINFO_RTD_PT200"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_PT500,         (char*)"CHINFO_RTD_PT500"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_PT1000,        (char*)"CHINFO_RTD_PT1000"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_JPT100,        (char*)"CHINFO_RTD_JPT100"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_JPT200,        (char*)"CHINFO_RTD_JPT200"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_JPT500,        (char*)"CHINFO_RTD_JPT500"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_JPT1000,       (char*)"CHINFO_RTD_JPT1000"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_NI100,         (char*)"CHINFO_RTD_NI100"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_NI200,         (char*)"CHINFO_RTD_NI200"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_NI500,         (char*)"CHINFO_RTD_NI500"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_NI1000,        (char*)"CHINFO_RTD_NI1000"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_RES_310,       (char*)"CHINFO_RTD_RES_310"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_RES_620,       (char*)"CHINFO_RTD_RES_620"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_RES_1250,      (char*)"CHINFO_RTD_RES_1250"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_RES_2200,      (char*)"CHINFO_RTD_RES_2200"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_PT50,          (char*)"CHINFO_RTD_PT50"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_RTD,        CHINFO_RTD_NI120,         (char*)"CHINFO_RTD_NI120"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_SERIAL_TAG, CHINFO_SERIALTAG_BYTE,    (char*)"CHINFO_SERIALTAG_BYTE"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_SERIAL_TAG, CHINFO_SERIALTAG_WORD,    (char*)"CHINFO_SERIALTAG_WORD"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_SERIAL_TAG, CHINFO_SERIALTAG_DWORD,   (char*)"CHINFO_SERIALTAG_DWORD"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_SERIAL_TAG, CHINFO_SERIALTAG_INT,     (char*)"CHINFO_SERIALTAG_INT"},
    {IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_SERIAL_TAG, CHINFO_SERIALTAG_FLOAT,   (char*)"CHINFO_SERIALTAG_FLOAT"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_WIR,	      CHINFO_IR_WORD,	    	(char*)"CHINFO_IR_WORD"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_FIR,		  CHINFO_IR_FLOAT,			(char*)"CHINFO_IR_FLOAT"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_MAXIMUM,		(char*)"CHINFO_VC_MAXIMUM"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_MINIMUM,		(char*)"CHINFO_VC_MINIMUM"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_AVERAGE,		(char*)"CHINFO_VC_AVERAGE"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_ACCUMULATION,	(char*)"CHINFO_VC_ACCUMULATION"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_INSTANTANEOUS,	(char*)"CHINFO_VC_INSTANTANEOUS"},
	{IOTYPE_SYS_IO_MODE, IOFUNC_SYS_ALL_CH_MODE_VC,		  CHINFO_VC_INCREMENTAL,	(char*)"CHINFO_VC_INCREMENTAL"},
};
//-----------------------------------------------------------------

BOOL PrintCHInfoStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp)
{
    BOOL bIsModeCmd = FALSE;
    WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
    WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;
    for(int i=0;i < sizeof(FuncInfo)/sizeof(FuncInfo[0]); i++)
    {
        if(FuncInfo[i].wIOType == wIOType)
        {
            if(FuncInfo[i].wFunc == wFunc)
            {
                bIsModeCmd = TRUE;
                if(FuncInfo[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
                {
                    printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
                        pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
                        FuncInfo[i].szChInfo);
                    return bIsModeCmd;
                }
            }
        }
    }
    if(bIsModeCmd)
    {
        printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
            pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    }
    return bIsModeCmd;
}

//-----------------------------------------------------------------
FUNC_CHINFO TemperatureUnit[] = {
	{IOTYPE_TC,       IOFUNC_TC_CH_TEMPERATURE_UNIT,             MXIO_TEMPERATURE_UNIT_C,                      (char*)"MXIO_TEMPERATURE_UNIT_C"},
	{IOTYPE_TC,       IOFUNC_TC_CH_TEMPERATURE_UNIT,             MXIO_TEMPERATURE_UNIT_F,                      (char*)"MXIO_TEMPERATURE_UNIT_F"},
	{IOTYPE_TC,       IOFUNC_TC_CH_TEMPERATURE_UNIT,             MXIO_TEMPERATURE_UNIT_MILLIVOLT,              (char*)"MXIO_TEMPERATURE_UNIT_MILLIVOLT"},
	{IOTYPE_TC,       IOFUNC_TC_CH_TEMPERATURE_UNIT,             MXIO_TEMPERATURE_UNIT_RESISTANCE,             (char*)"MXIO_TEMPERATURE_UNIT_RESISTANCE"},
	{IOTYPE_RTD,      IOFUNC_RTD_CH_TEMPERATURE_UNIT,            MXIO_TEMPERATURE_UNIT_C,                      (char*)"MXIO_TEMPERATURE_UNIT_C"},
	{IOTYPE_RTD,      IOFUNC_RTD_CH_TEMPERATURE_UNIT,            MXIO_TEMPERATURE_UNIT_F,                      (char*)"MXIO_TEMPERATURE_UNIT_F"},
	{IOTYPE_RTD,      IOFUNC_RTD_CH_TEMPERATURE_UNIT,            MXIO_TEMPERATURE_UNIT_MILLIVOLT,              (char*)"MXIO_TEMPERATURE_UNIT_MILLIVOLT"},
	{IOTYPE_RTD,      IOFUNC_RTD_CH_TEMPERATURE_UNIT,            MXIO_TEMPERATURE_UNIT_RESISTANCE,             (char*)"MXIO_TEMPERATURE_UNIT_RESISTANCE"},
};

//-----------------------------------------------------------------
BOOL PrintTemperatureUnitStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp)
{
	BOOL bIsModeCmd = FALSE;
	WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
	WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;

	for(int i=0;i < sizeof(TemperatureUnit)/sizeof(TemperatureUnit[0]); i++)
	{
		if(TemperatureUnit[i].wIOType == wIOType)
		{
			if(TemperatureUnit[i].wFunc == wFunc)
			{
				bIsModeCmd = TRUE;
				if(TemperatureUnit[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
				{
					printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
						pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
						TemperatureUnit[i].szChInfo);
					return bIsModeCmd;
				}
			}
		}
	}
	if(bIsModeCmd)
	{
		printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
			pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	}
	return bIsModeCmd;
}


FUNC_CHINFO CounterActiveMode[] = {
	{IOTYPE_DI,       IOFUNC_DI_COUNTER_CH_ACTIVE_MODE,             MXIO_COUNTER_ACTIVE_MODE_BOTH,                      (char*)"MXIO_COUNTER_ACTIVE_MODE_BOTH"},
	{IOTYPE_DI,       IOFUNC_DI_COUNTER_CH_ACTIVE_MODE,             MXIO_COUNTER_ACTIVE_MODE_RISING_EDGE,               (char*)"MXIO_COUNTER_ACTIVE_MODE_RISING_EDGE"},
	{IOTYPE_DI,       IOFUNC_DI_COUNTER_CH_ACTIVE_MODE,             MXIO_COUNTER_ACTIVE_MODE_FALLING_EDGE,              (char*)"MXIO_COUNTER_ACTIVE_MODE_FALLING_EDGE"},
	{IOTYPE_DIO,      IOFUNC_DIO_COUNTER_CH_ACTIVE_MODE,            MXIO_COUNTER_ACTIVE_MODE_BOTH,                      (char*)"MXIO_COUNTER_ACTIVE_MODE_BOTH"},
	{IOTYPE_DIO,      IOFUNC_DIO_COUNTER_CH_ACTIVE_MODE,            MXIO_COUNTER_ACTIVE_MODE_RISING_EDGE,               (char*)"MXIO_COUNTER_ACTIVE_MODE_RISING_EDGE"},
	{IOTYPE_DIO,      IOFUNC_DIO_COUNTER_CH_ACTIVE_MODE,            MXIO_COUNTER_ACTIVE_MODE_FALLING_EDGE,              (char*)"MXIO_COUNTER_ACTIVE_MODE_FALLING_EDGE"},
};

//-----------------------------------------------------------------
BOOL PrintCounterActiveModeStr(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp)
{
	BOOL bIsModeCmd = FALSE;
	WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
	WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;

	for(int i=0;i < sizeof(CounterActiveMode)/sizeof(CounterActiveMode[0]); i++)
	{
		if(CounterActiveMode[i].wIOType == wIOType)
		{
			if(CounterActiveMode[i].wFunc == wFunc)
			{
				bIsModeCmd = TRUE;
				if(CounterActiveMode[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
				{
					printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
						pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
						CounterActiveMode[i].szChInfo);
					return bIsModeCmd;
				}
			}
		}
	}
	if(bIsModeCmd)
	{
		printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
			pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	}
	return bIsModeCmd;
}
//-----------------------------------------------------------------
void PrintReadValue(int iIndex, CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pImp)
{
    if(PrintCHInfoStr(iIndex, pImp))
        return;
	if(PrintTemperatureUnitStr(iIndex, pImp))
		return;
	if(PrintCounterActiveModeStr(iIndex, pImp))
		return;
    BYTE bytDataType = pImp->pRwChannelData[iIndex].bytDataType;
    BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;
    if ((DATA_TYPE_BYTE == bytDataType) || (DATA_TYPE_WORD == bytDataType))
        printf("Slot%d, CH[%d] = %d(0x%02X)\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    else if ((DATA_TYPE_DWORD == bytDataType) || (DATA_TYPE_INT == bytDataType))
        printf("Slot%d, CH[%d] = %d(0x%04X)\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    else if (DATA_TYPE_FLOAT == bytDataType)
        printf("Slot%d, CH[%d] = %.3f\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.fValue);
    else if (DATA_TYPE_STRING == bytDataType)
        printf("Slot%d, CH[%d] = %s\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.szValue);
    else if (DATA_TYPE_DATE == bytDataType)
    {
        printf("Slot%d, CH[%d] = %d/%d/%d\n", bytSlotNum, iIndex,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Year,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Month,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Day);
    }
    else if (DATA_TYPE_TIME == bytDataType)
    {
        printf("Slot%d, CH[%d] = %d:%d:%d\n", bytSlotNum, iIndex,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Hour,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Min,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Sec);
    }
    else if (DATA_TYPE_UNKNOWN == bytDataType)
        printf("Slot%d, CH[%d] = data type return DATA_TYPE_UNKNOWN\n", bytSlotNum, iIndex);
    else
        printf("Slot%d, CH[%d] = not defined data type = %d\n", bytSlotNum, iIndex, bytDataType);
}
//-----------------------------------------------------------------
void CALLBACK ClearDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvClearBlockData, void *pParam)
{
	CMXIO_V3_2500::MX_RECV_CLEAR_BLOCK_DATA *pClearImp = (CMXIO_V3_2500::MX_RECV_CLEAR_BLOCK_DATA *)pRecvClearBlockData;
	if (pClearImp)
	{
		if (MXIO_V3_BLOCK_OK == pClearImp->MxRecvBlockHeader.wResult)
		{
			printf("Clear Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pClearImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				PrtChErrorString(i, pClearImp->pMxClearChannelData[i].bytErrorCode);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pClearImp->MxRecvBlockHeader.wResult);
	}
}

//-----------------------------------------------------------------
void CALLBACK WriteDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvWriteBlockData, void *pParam)
{
	CMXIO_V3_2500::MX_RECV_WRITE_BLOCK_DATA *pWriteImp = (CMXIO_V3_2500::MX_RECV_WRITE_BLOCK_DATA *)pRecvWriteBlockData;
	if (pWriteImp)
	{
		if (MXIO_V3_BLOCK_OK == pWriteImp->MxRecvBlockHeader.wResult) //
		{
			printf("Write Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pWriteImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				PrtChErrorString(i, pWriteImp->pRwChannelData[i].bytErrorCode);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pWriteImp->MxRecvBlockHeader.wResult);
	}
}
//-----------------------------------------------------------------
void CALLBACK ReadDataCallbackFunc_2500(WORD wBlockQuantity, int *pRecvReadBlockData, void *pParam)
{
	CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *pReadImp = (CMXIO_V3_2500::MX_RECV_READ_BLOCK_DATA *)pRecvReadBlockData;
	if (pReadImp)
	{
		if (MXIO_V3_BLOCK_OK == pReadImp->MxRecvBlockHeader.wResult) //
		{
			printf("Read Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pReadImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				if (MXIO_V3_CH_OK == pReadImp->pRwChannelData[i].bytErrorCode)
					PrintReadValue(i, pReadImp);
				else
					PrtStrChErrorString(pReadImp->pRwChannelData[i].bytErrorCode, pReadImp->MxRecvStrCmd->szDevChName);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pReadImp->MxRecvBlockHeader.wResult);
	}
}
//-----------------------------------------------------------------
void *Thread_2500( void * lParam )
{
	DEVICE_PARAMETERS *io = (DEVICE_PARAMETERS *)lParam;
	CMXIO_V3_2500::MX_MAC_ADDRESS MxMACAddr;
	CMXIO_V3_2500::MX_ACCOUNT_DATA AccountData;
	strcpy(AccountData.szUserName, io->szUserName);
	strcpy(AccountData.szPwd, io->szPwd);
    char bytInputValue[64] = {'\0'};
    int iRet = MXIO_V3_OK;

    iRet = g_pMXIO_V3_2500->MXIO_Init();
    printf("Thread_2500: MXIO_Init() return %d(0x%X)\n", iRet, iRet);
    //----------------------------------------------------------------
    //
	int ihConnection = 0, i = 0;
	char szTmp[18] = {'\0'};
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
	if(io->szMACAddr)
	{
		strcpy(szTmp, io->szMACAddr);
		char* pch = strtok (szTmp,".-");
		while (pch != NULL)
		{
			MxMACAddr.bytMAC[i] = (BYTE)str_to_hex(pch);
			pch = strtok (NULL, ".-");
			i++;
		}
	}
	printf("Thread_2500: IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
		io->IPAddress, io->szMACAddr, io->wPort, io->szUserName, io->szPwd);
	iRet = g_pMXIO_V3_2500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(MxMACAddr), &AccountData, &ihConnection);
	CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	printf("Thread_2500: IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
	//-----------------------------------------------------
	while(FALSE == io->bStop)
	{
        iRet = MXIO_V3_OK; //initialize

		//Read
		if (MXIO_V3_OK == iRet)
		{
			WORD wStartObj = 0, wTotalObj = 8;
			CMXIO_V3_2500::MX_READ_BLOCK_DATA ReadBlockDataArray[BLOCK_SIZE];
			for (int i = 0; i < BLOCK_SIZE; i++)
			{
				printf("Thread_2500: Slot Num: 0, IOType_SubType Num: %d, Function Code: %d, Start Ch: 0, Total Ch: 1\n",
					IOTYPE_SYS_SYSTEM, IOFUNC_SYS_MODEL_NAME);
				ReadBlockDataArray[i].MxCmdBlockHeader.bytSlotNum = (BYTE)(0);
				ReadBlockDataArray[i].MxCmdBlockHeader.wIOType = (WORD)(IOTYPE_SYS_SYSTEM);
				ReadBlockDataArray[i].MxCmdBlockHeader.wFunction = (WORD)(IOFUNC_SYS_MODEL_NAME);
				ReadBlockDataArray[i].MxCmdBlockHeader.wStartObj = (WORD)(0);//wStartObj;
				ReadBlockDataArray[i].MxCmdBlockHeader.wTotalObj = (WORD)(1);//wTotalObj;
			}
			//-----------------------
			if ((MXIO_V3_OK == iRet) && (0 == ihConnection))
			{
				printf("Thread_2500: IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
					io->IPAddress, io->szMACAddr, io->wPort, AccountData.szUserName, AccountData.szPwd);
				iRet = g_pMXIO_V3_2500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(MxMACAddr), &AccountData, &ihConnection);
				CheckErr(ihConnection, iRet, (char*)"Thread_2500: MXIO_Connect");
				printf("Thread_2500: IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
			}
			printf("Thread_2500: MXIO_Read(%d)\n", ihConnection);
			iRet = g_pMXIO_V3_2500->MXIO_Read(ihConnection, BLOCK_SIZE, ReadBlockDataArray, ReadDataCallbackFunc_2500, NULL);
			CheckErr(ihConnection, iRet, (char*)"Thread_2500: MXIO_Read");
		}
		sleep(1);
	}

	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_2500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"Thread_2500: MXIO_Disconnect");
	}    
    return 0;
}

//-----------------------------------------------------------------
//ioLogik 4500 Series
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName)
{
	phMXIO_V3_4500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_4500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_4500 pfnCreateMXIO_V3_4500 = (PFNCREATEMXIO_V3_4500)dlsym(phMXIO_V3_4500, "CreateMXIO_4500");
	if (NULL == pfnCreateMXIO_V3_4500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_4500 = pfnCreateMXIO_V3_4500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500)
{
	PFNDESTROYMXIO_V3_4500 pfnDestroyMXIO_V3_4500 = (PFNDESTROYMXIO_V3_4500)dlsym(hMXIO_V3_4500, "DestroyMXIO_4500");
	if (!pfnDestroyMXIO_V3_4500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_4500(*pMXIO_V3_4500);
	dlclose(hMXIO_V3_4500);
	*pMXIO_V3_4500 = NULL;
	return 0;
}
//-----------------------------------------------------------------

BOOL PrintCHInfoStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp)
{
    BOOL bIsModeCmd = FALSE;
    WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
    WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;
    for(int i=0;i < sizeof(FuncInfo)/sizeof(FuncInfo[0]); i++)
    {
        if(FuncInfo[i].wIOType == wIOType)
        {
            if(FuncInfo[i].wFunc == wFunc)
            {
                bIsModeCmd = TRUE;
                if(FuncInfo[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
                {
                    printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
                        pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
                        FuncInfo[i].szChInfo);
                    return bIsModeCmd;
                }
            }
        }
    }
    if(bIsModeCmd)
    {
        printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
            pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    }
    return bIsModeCmd;
}

//-----------------------------------------------------------------
BOOL PrintTemperatureUnitStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp)
{
	BOOL bIsModeCmd = FALSE;
	WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
	WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;

	for(int i=0;i < sizeof(TemperatureUnit)/sizeof(TemperatureUnit[0]); i++)
	{
		if(TemperatureUnit[i].wIOType == wIOType)
		{
			if(TemperatureUnit[i].wFunc == wFunc)
			{
				bIsModeCmd = TRUE;
				if(TemperatureUnit[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
				{
					printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
						pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
						TemperatureUnit[i].szChInfo);
					return bIsModeCmd;
				}
			}
		}
	}
	if(bIsModeCmd)
	{
		printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
			pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	}
	return bIsModeCmd;
}

//-----------------------------------------------------------------
BOOL PrintCounterActiveModeStr(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp)
{
	BOOL bIsModeCmd = FALSE;
	WORD wIOType = pImp->MxRecvBlockHeader.wIOType;
	WORD wFunc = pImp->MxRecvBlockHeader.wFunction;
	BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;

	for(int i=0;i < sizeof(CounterActiveMode)/sizeof(CounterActiveMode[0]); i++)
	{
		if(CounterActiveMode[i].wIOType == wIOType)
		{
			if(CounterActiveMode[i].wFunc == wFunc)
			{
				bIsModeCmd = TRUE;
				if(CounterActiveMode[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
				{
					printf("Slot%d ch[%d] Mode = %d(0x%X): %s\r\n", bytSlotNum, iIndex,
						pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
						CounterActiveMode[i].szChInfo);
					return bIsModeCmd;
				}
			}
		}
	}
	if(bIsModeCmd)
	{
		printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
			pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	}
	return bIsModeCmd;
}
//-----------------------------------------------------------------
void PrintReadValue(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp)
{
    if(PrintCHInfoStr(iIndex, pImp))
        return;
	if(PrintTemperatureUnitStr(iIndex, pImp))
		return;
	if(PrintCounterActiveModeStr(iIndex, pImp))
		return;
    BYTE bytDataType = pImp->pRwChannelData[iIndex].bytDataType;
    BYTE bytSlotNum = pImp->MxRecvBlockHeader.bytSlotNum;
    if ((DATA_TYPE_BYTE == bytDataType) || (DATA_TYPE_WORD == bytDataType))
        printf("Slot%d, CH[%d] = %d(0x%02X)\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    else if ((DATA_TYPE_DWORD == bytDataType) || (DATA_TYPE_INT == bytDataType))
        printf("Slot%d, CH[%d] = %d(0x%04X)\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
    else if (DATA_TYPE_FLOAT == bytDataType)
        printf("Slot%d, CH[%d] = %.3f\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.fValue);
    else if (DATA_TYPE_STRING == bytDataType)
        printf("Slot%d, CH[%d] = %s\n", bytSlotNum, iIndex, pImp->pRwChannelData[iIndex].unValue.szValue);
    else if (DATA_TYPE_DATE == bytDataType)
    {
        printf("Slot%d, CH[%d] = %d/%d/%d\n", bytSlotNum, iIndex,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Year,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Month,
            pImp->pRwChannelData[iIndex].unValue.MxDate.Day);
    }
    else if (DATA_TYPE_TIME == bytDataType)
    {
        printf("Slot%d, CH[%d] = %d:%d:%d\n", bytSlotNum, iIndex,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Hour,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Min,
            pImp->pRwChannelData[iIndex].unValue.MxTime.Sec);
    }
    else if (DATA_TYPE_UNKNOWN == bytDataType)
        printf("Slot%d, CH[%d] = data type return DATA_TYPE_UNKNOWN\n", bytSlotNum, iIndex);
    else
        printf("Slot%d, CH[%d] = not defined data type = %d\n", bytSlotNum, iIndex, bytDataType);
}
//-----------------------------------------------------------------
void CALLBACK ClearDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvClearBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_CLEAR_BLOCK_DATA *pClearImp = (CMXIO_V3_4500::MX_RECV_CLEAR_BLOCK_DATA *)pRecvClearBlockData;
	if (pClearImp)
	{
		if (MXIO_V3_BLOCK_OK == pClearImp->MxRecvBlockHeader.wResult)
		{
			printf("Clear Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pClearImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				PrtChErrorString(i, pClearImp->pMxClearChannelData[i].bytErrorCode);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pClearImp->MxRecvBlockHeader.wResult);
	}
}

//-----------------------------------------------------------------
void CALLBACK WriteDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvWriteBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_WRITE_BLOCK_DATA *pWriteImp = (CMXIO_V3_4500::MX_RECV_WRITE_BLOCK_DATA *)pRecvWriteBlockData;
	if (pWriteImp)
	{
		if (MXIO_V3_BLOCK_OK == pWriteImp->MxRecvBlockHeader.wResult) //
		{
			printf("Write Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pWriteImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				PrtChErrorString(i, pWriteImp->pRwChannelData[i].bytErrorCode);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pWriteImp->MxRecvBlockHeader.wResult);
	}
}
//-----------------------------------------------------------------
void CALLBACK ReadDataCallbackFunc_4500(WORD wBlockQuantity, int *pRecvReadBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pReadImp = (CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *)pRecvReadBlockData;
	if (pReadImp)
	{
		if (MXIO_V3_BLOCK_OK == pReadImp->MxRecvBlockHeader.wResult) //
		{
			printf("Read Block[%d] Success:\n", (wBlockQuantity-1));
			for (int i = 0; i < pReadImp->MxRecvBlockHeader.wTotalObj; i++)
			{
				if (MXIO_V3_CH_OK == pReadImp->pRwChannelData[i].bytErrorCode)
					PrintReadValue(i, pReadImp);
				else
					PrtStrChErrorString(pReadImp->pRwChannelData[i].bytErrorCode, pReadImp->MxRecvStrCmd->szDevChName);
			}
		}
		else
			PrtBlockErrorString((wBlockQuantity - 1), pReadImp->MxRecvBlockHeader.wResult);
	}
}

//-----------------------------------------------------------------
void *Thread_4500( void * lParam )
{
	DEVICE_PARAMETERS *io = (DEVICE_PARAMETERS *)lParam;
	CMXIO_V3_4500::MX_MAC_ADDRESS MxMACAddr;
	CMXIO_V3_4500::MX_ACCOUNT_DATA AccountData;
	strcpy(AccountData.szUserName, io->szUserName);
	strcpy(AccountData.szPwd, io->szPwd);
    char bytInputValue[64] = {'\0'};
    int iRet = MXIO_V3_OK;

    iRet = g_pMXIO_V3_4500->MXIO_Init();
    printf("Thread_4500: MXIO_Init() return %d(0x%X)\n", iRet, iRet);
    //----------------------------------------------------------------
    //
	int ihConnection = 0, i = 0;
	char szTmp[18] = {'\0'};
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
	if(io->szMACAddr)
	{
		strcpy(szTmp, io->szMACAddr);
		char* pch = strtok (szTmp,".-");
		while (pch != NULL)
		{
			MxMACAddr.bytMAC[i] = (BYTE)str_to_hex(pch);
			pch = strtok (NULL, ".-");
			i++;
		}
	}
	printf("Thread_4500: IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
		io->IPAddress, io->szMACAddr, io->wPort, io->szUserName, io->szPwd);
	iRet = g_pMXIO_V3_4500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(MxMACAddr), &AccountData, &ihConnection);
	CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	printf("Thread_4500: IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
	//-----------------------------------------------------
	while(FALSE == io->bStop)
	{
        iRet = MXIO_V3_OK; //initialize

		//Read
		if (MXIO_V3_OK == iRet)
		{
			WORD wStartObj = 0, wTotalObj = 8;
			CMXIO_V3_4500::MX_READ_BLOCK_DATA ReadBlockDataArray[BLOCK_SIZE];
			for (int i = 0; i < BLOCK_SIZE; i++)
			{
				printf("Thread_4500: Slot Num: 0, IOType_SubType Num: %d, Function Code: %d, Start Ch: 0, Total Ch: 1\n",
					IOTYPE_SYS_SYSTEM, IOFUNC_SYS_SERVER_NAME);
				ReadBlockDataArray[i].MxCmdBlockHeader.bytSlotNum = (BYTE)(0);
				ReadBlockDataArray[i].MxCmdBlockHeader.wIOType = (WORD)(IOTYPE_SYS_SYSTEM);
				ReadBlockDataArray[i].MxCmdBlockHeader.wFunction = (WORD)(IOFUNC_SYS_SERVER_NAME);
				ReadBlockDataArray[i].MxCmdBlockHeader.wStartObj = (WORD)(0);//wStartObj;
				ReadBlockDataArray[i].MxCmdBlockHeader.wTotalObj = (WORD)(1);//wTotalObj;
			}
			//-----------------------
			if ((MXIO_V3_OK == iRet) && (0 == ihConnection))
			{
				printf("Thread_4500: IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
					io->IPAddress, io->szMACAddr, io->wPort, AccountData.szUserName, AccountData.szPwd);
				iRet = g_pMXIO_V3_4500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(MxMACAddr), &AccountData, &ihConnection);
				CheckErr(ihConnection, iRet, (char*)"Thread_4500: MXIO_Connect");
				printf("Thread_4500: IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
			}
			printf("Thread_4500: MXIO_Read(%d)\n", ihConnection);
			iRet = g_pMXIO_V3_4500->MXIO_Read(ihConnection, BLOCK_SIZE, ReadBlockDataArray, ReadDataCallbackFunc_4500, NULL);
			CheckErr(ihConnection, iRet, (char*)"Thread_4500: MXIO_Read");
		}
		sleep(1);
	}

	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"Thread_4500: MXIO_Disconnect");
	}    
    return 0;
}
//-----------------------------------------------------------------
