/*******************************************************************************
 * Copyright Moxa Inc.
 *
 *
 * Date          Author            Comment
 * 02/25/2019    Daniel Chang      Created.
 ******************************************************************************/

 /*!
    \file LIBDateVersion.cpp
    \copyright Moxa Inc.
    \brief <b>\a Get \a LIB \a Date \a and \a Version</b>
    \date 02/25/2019
    \author Daniel Chang
    \version V1.0
    \details
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <MXIO_V3_4500.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
//-----------------------------------------------------------------
#define TRUE    1
#define FALSE   0

#define HMODULE void
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500);
char getch();
void WaitPressAnyKey();
//-----------------------------------------------------------------
HMODULE *g_hMXIO_V3_4500 = NULL;
CMXIO_V3_4500 *g_pMXIO_V3_4500 = NULL;

//-----------------------------------------------------------------
//! \include LIBDateVersion.cpp
int main(int argc, char *argv[])
{
	char bytInputValue[64] = {'\0'};
	char szIP[] = "192.168.127.253";
	CMXIO_V3_4500::MX_MAC_ADDRESS MxMACAddr;
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
    char szMACAddr[18] = {'\0'};//"00-90-e8-00-00-41";
	DWORD dwTimeout = 3000;
	int ihConnection = 0;
	CMXIO_V3_4500::MX_ACCOUNT_DATA AccountData;
	sprintf(AccountData.szUserName,"%s", (char*)"admin");
	sprintf(AccountData.szPwd,"%s", (char*)"moxa");

	//-----------------------------------------------
	char IPAddress[16] = {'\0'};
	char szTempBuf[16] = {'\0'};
	WORD wPort = MXIO_V3_AGENT_PORT;
	sprintf(IPAddress, "%s", szIP);
	if (argc <= 1)
	{
		printf("Cmd Format: [IP Address] [Timeout] [TCP Port]\n");
		strcpy(IPAddress, szIP);
	}
	//
	if (argc > 1)
		sprintf(IPAddress, "%s", argv[1]);
	if (argc > 2)
		dwTimeout = atoi(argv[2]);
	if (argc > 3)
		wPort = atoi(argv[3]);
	printf("Input IP Address=%s, TimeOut=%d, TCP Port=%d\n", IPAddress, dwTimeout, wPort);
	//-----------------------
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500.so");
#endif
	printf("MXCreateLibrary() g_pMXIO_V3_4500=0x%p return %d(0x%X)\n", g_pMXIO_V3_4500, iRet, iRet);
	//-----------------------------------------------------
	if(MXIO_V3_OK == iRet)
	{
		//LIB Version
		CMXIO_V3_4500::MX_VERSION MxVersion = {'\0'};
		g_pMXIO_V3_4500->MXIO_GetDllVersion(&MxVersion);
		printf("LIB Version: %d.%d.%d.%d\n",
			MxVersion.Major, MxVersion.Minor, MxVersion.Beta, MxVersion.Revision);
		//LIB Build Date
		CMXIO_V3_4500::MX_DATE MxDate = {'\0'};
		g_pMXIO_V3_4500->MXIO_GetDllBuildDate(&MxDate);
		printf("LIB Build Date: %d-%d-%d\n", MxDate.Year, MxDate.Month, MxDate.Day);
	}
	//-----------------------
	if (NULL != g_hMXIO_V3_4500)
	{
		iRet = MXDestroyLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500);
		printf("MXDestroyLibrary() return %d(0x%X)\n", iRet, iRet);
	}
	WaitPressAnyKey();
	return 0;
}
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName)
{
	phMXIO_V3_4500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_4500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_4500 pfnCreateMXIO_V3_4500 = (PFNCREATEMXIO_V3_4500)dlsym(phMXIO_V3_4500, "CreateMXIO_4500");
	if (NULL == pfnCreateMXIO_V3_4500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_4500 = pfnCreateMXIO_V3_4500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500)
{
	PFNDESTROYMXIO_V3_4500 pfnDestroyMXIO_V3_4500 = (PFNDESTROYMXIO_V3_4500)dlsym(hMXIO_V3_4500, "DestroyMXIO_4500");
	if (!pfnDestroyMXIO_V3_4500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_4500(*pMXIO_V3_4500);
	dlclose(hMXIO_V3_4500);
	*pMXIO_V3_4500 = NULL;
	return 0;
}
//-----------------------------------------------------------------
char getch()
{
    int iRet;
	char ch;
	struct termios oldt, newt;

	tcgetattr(STDIN_FILENO, &oldt);
	memcpy(&newt, &oldt, sizeof(newt));
	newt.c_lflag &= ~( ECHO | ICANON | ECHOE | ECHOK |
						ECHONL | ECHOPRT | ECHOKE | ICRNL);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	iRet = read(STDIN_FILENO, &ch, 1);
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if(iRet == 1)
		return ch;
	else
		return 0;
}

void WaitPressAnyKey()
{
	printf("press any key to exit.\n");
	getch();
}

//-----------------------------------------------------------------
