/*******************************************************************************
 * Copyright Moxa Inc.
 *
 *
 * Date          Author            Comment
 * 02/25/2019    Daniel Chang      Created.
 ******************************************************************************/

 /*!
    \file MultiThread.cpp
    \copyright Moxa Inc.
    \brief <b>\a Connect \a Multi \a Devices \a by \a Thread</b>
    \date 02/25/2019
    \author Daniel Chang
    \version V1.0
    \details
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <MXIO_V3_4500.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>

#include <pthread.h>

#define HANDLE  int
#define TRUE    1
#define FALSE   0

#define HMODULE void
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500);
char getch();
void WaitPressAnyKey();
void PrtStrErrorString(WORD wErrorCode, char *szName);
void PrtStrChErrorString(BYTE bytErrorCode, char* pszName);

void CALLBACK ReadStrDataCallbackFunc1(int *pRecvReadBlockData, void *pParam);
void CALLBACK ReadStrDataCallbackFunc2(int *pRecvReadBlockData, void *pParam);
void PrintStrReadValue(int iIndex, CMXIO_V3_4500::MX_RECV_STR_READ_DATA *pImp);

void *ThreadRead1( void * lParam );
void *ThreadRead2( void * lParam );
//-----------------------------------------------------------------
// For thread used
typedef struct _DEVICE_PARAMETERS{
	char IPAddress[16];
	WORD wPort;
	DWORD dwTimeout;
	char szMACAddr[18];
	CMXIO_V3_4500::MX_MAC_ADDRESS MxMACAddr;
	CMXIO_V3_4500::MX_ACCOUNT_DATA *AccountData;
	bool bStop;
}DEVICE_PARAMETERS;
//-----------------------------------------------------------------
HMODULE *g_hMXIO_V3_4500 = NULL;
CMXIO_V3_4500 *g_pMXIO_V3_4500 = NULL;

//-----------------------------------------------------------------
//! \include MultiThread.cpp
int main(int argc, char *argv[])
{
	char bytInputValue[64] = {'\0'};
	char szIP[] = "192.168.127.253";
	char szIP2[] = "192.168.127.250";
	CMXIO_V3_4500::MX_MAC_ADDRESS MxMACAddr;
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
    char szMACAddr[18] = {'\0'};//"00-90-e8-00-00-41";
	DWORD dwTimeout = 3000;
	int ihConnection = 0;
	int ihConnection2 = 0;
	CMXIO_V3_4500::MX_ACCOUNT_DATA AccountData;
	sprintf(AccountData.szUserName,"%s", (char*)"admin");
	sprintf(AccountData.szPwd,"%s", (char*)"moxa");

	//-----------------------------------------------
	char IPAddress[16] = {'\0'};
	char IPAddress2[16] = {'\0'};
	char szTempBuf[16] = {'\0'};
	WORD wPort = MXIO_V3_AGENT_PORT;
	sprintf(IPAddress, "%s", szIP);
	if (argc <= 1)
	{
		printf("Cmd Format: [IP Address] [Timeout] [TCP Port] [IP Address2]\n");
		strcpy(IPAddress, szIP);
		strcpy(IPAddress2, szIP2);
	}
	//
	if (argc > 1)
		sprintf(IPAddress, "%s", argv[1]);
	if (argc > 2)
		dwTimeout = atoi(argv[2]);
	if (argc > 3)
		wPort = atoi(argv[3]);
	if (argc > 4)
		sprintf(IPAddress2, "%s", argv[4]);
	printf("Input IP Address=%s, TimeOut=%d, TCP Port=%d, IP Address2=%s\n",
		IPAddress, dwTimeout, wPort, IPAddress2);
	//-----------------------
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500.so");
#endif
	printf("MXCreateLibrary() g_pMXIO_V3_4500=0x%p return %d(0x%X)\n", g_pMXIO_V3_4500, iRet, iRet);
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Init();
		printf("MXIO_Init() return %d(0x%X)\n", iRet, iRet);
	}
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
#define THREAD_QUANTITY		2
		HANDLE hThread[THREAD_QUANTITY];
	    pthread_t dwthread[THREAD_QUANTITY];
	    int iDevice = (sizeof(dwthread)/sizeof(dwthread[0]));
	    DEVICE_PARAMETERS *DeviceParameters = new DEVICE_PARAMETERS[iDevice];

		//Initial
		for(int i=0; i < iDevice; i++)
		{
			DeviceParameters[i].wPort = wPort;
			DeviceParameters[i].dwTimeout = dwTimeout;
			memcpy(DeviceParameters[i].MxMACAddr.bytMAC, MxMACAddr.bytMAC, sizeof(BYTE)*6);
			DeviceParameters[i].AccountData = &AccountData;
			DeviceParameters[i].bStop = FALSE;
		}
		//Thread [0]
		strcpy(DeviceParameters[0].IPAddress, IPAddress);
		hThread[0] = pthread_create( &dwthread[0], 0, ThreadRead1, (void *)&DeviceParameters[0]);
		//Thread [1]
		strcpy(DeviceParameters[1].IPAddress, IPAddress2);
		hThread[1] = pthread_create( &dwthread[1], 0, ThreadRead2, (void *)&DeviceParameters[1]);

		// allow the thread to do some work:
		for (int i = 10; i > 0; i--)
		{
			usleep(1000000);
			printf("main thread wait {%d} Seconds\n", i);
		}
		//Stop thread
		for (int i = 0; i < iDevice; i++)
			DeviceParameters[i].bStop = TRUE;

		//wait thread finish
		for (int i = 0; i < iDevice; i++)
		{
		    iRet = pthread_join(dwthread[i], NULL);
			printf("pthread_join [%d] Tread return %d(0x%X)\n", i, iRet, iRet);
		}
		//Free memory
		delete [] DeviceParameters;
	}
	//-----------------------
	if (NULL != g_hMXIO_V3_4500)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Exit();
		printf("MXIO_Exit() return %d(0x%X)\n", iRet, iRet);
		iRet = MXDestroyLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500);
		printf("MXDestroyLibrary() return %d(0x%X)\n", iRet, iRet);
	}
	WaitPressAnyKey();
	return 0;
}

//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName)
{
	phMXIO_V3_4500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_4500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_4500 pfnCreateMXIO_V3_4500 = (PFNCREATEMXIO_V3_4500)dlsym(phMXIO_V3_4500, "CreateMXIO_4500");
	if (NULL == pfnCreateMXIO_V3_4500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_4500 = pfnCreateMXIO_V3_4500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500)
{
	PFNDESTROYMXIO_V3_4500 pfnDestroyMXIO_V3_4500 = (PFNDESTROYMXIO_V3_4500)dlsym(hMXIO_V3_4500, "DestroyMXIO_4500");
	if (!pfnDestroyMXIO_V3_4500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_4500(*pMXIO_V3_4500);
	dlclose(hMXIO_V3_4500);
	*pMXIO_V3_4500 = NULL;
	return 0;
}
//-----------------------------------------------------------------
char getch()
{
    int iRet;
	char ch;
	struct termios oldt, newt;

	tcgetattr(STDIN_FILENO, &oldt);
	memcpy(&newt, &oldt, sizeof(newt));
	newt.c_lflag &= ~( ECHO | ICANON | ECHOE | ECHOK |
						ECHONL | ECHOPRT | ECHOKE | ICRNL);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	iRet = read(STDIN_FILENO, &ch, 1);
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if(iRet == 1)
		return ch;
	else
		return 0;
}

void WaitPressAnyKey()
{
	printf("press any key to exit.\n");
	getch();
}

//-----------------------------------------------------------------
//===========================================================================
void CheckErr(int ihConnection, int iRet, char *szFunctionName)
{
	char *szErrMsg = (char*)"MXIO_V3_OK";
	if (iRet != MXIO_V3_OK)
	{
		switch (iRet)
		{
		case MXIO_V3_ILLEGAL_FUNCTION:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_FUNCTION";
			break;
		case MXIO_V3_ILLEGAL_DATA_ADDRESS:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_ADDRESS";
			break;
		case MXIO_V3_ILLEGAL_DATA_VALUE:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_VALUE";
			break;
		case MXIO_V3_SLAVE_DEVICE_FAILURE:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_FAILURE";
			break;
		case MXIO_V3_SLAVE_DEVICE_BUSY:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_BUSY";
			break;
		case MXIO_V3_EIO_TIME_OUT:
			szErrMsg = (char*)"MXIO_V3_EIO_TIME_OUT";
			break;
		case MXIO_V3_EIO_INIT_SOCKETS_FAIL:
			szErrMsg = (char*)"MXIO_V3_EIO_INIT_SOCKETS_FAIL";
			break;
		case MXIO_V3_EIO_CREATING_SOCKET_ERROR:
			szErrMsg = (char*)"MXIO_V3_EIO_CREATING_SOCKET_ERROR";
			break;
		case MXIO_V3_EIO_RESPONSE_BAD:
			szErrMsg = (char*)"MXIO_V3_EIO_RESPONSE_BAD";
			break;
		case MXIO_V3_EIO_SOCKET_DISCONNECT:
			szErrMsg = (char*)"MXIO_V3_EIO_SOCKET_DISCONNECT";
			break;
		case MXIO_V3_PROTOCOL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_PROTOCOL_TYPE_ERROR";
			break;
		case MXIO_V3_PRODUCT_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_PRODUCT_NOT_SUPPORT";
			break;
		case MXIO_V3_HANDLE_ERROR:
		    szErrMsg = (char*)"MXIO_V3_HANDLE_ERROR";
		    break;
		case MXIO_V3_SLOT_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_SLOT_OUT_OF_RANGE";
			break;
		case MXIO_V3_CHANNEL_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_CHANNEL_OUT_OF_RANGE";
			break;
		case MXIO_V3_COIL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_COIL_TYPE_ERROR";
			break;
		case MXIO_V3_REGISTER_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_REGISTER_TYPE_ERROR";
			break;
		case MXIO_V3_FUNCTION_NOT_SUPPORT:
		    szErrMsg = (char*)"MXIO_V3_FUNCTION_NOT_SUPPORT";
		    break;
		case MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_INPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_INPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_EIO_PASSWORD_INCORRECT:
			szErrMsg = (char*)"MXIO_V3_EIO_PASSWORD_INCORRECT";
			break;
		case MXIO_V3_SLOT_NOT_EXIST:
			szErrMsg = (char*)"MXIO_V3_SLOT_NOT_EXIST";
			break;
		case MXIO_V3_FIRMWARE_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_FIRMWARE_NOT_SUPPORT";
			break;
		case MXIO_V3_CREATE_MUTEX_FAIL:
			szErrMsg = (char*)"MXIO_V3_CREATE_MUTEX_FAIL";
			break;
		case MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_FUNCTION_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_ERR_FUNCTION_NOT_SUPPORT";
			break;
		case MXIO_V3_ERR_INVALID_HANDLE:
			szErrMsg = (char*)"MXIO_V3_ERR_INVALID_HANDLE";
			break;
		case MXIO_V3_ERR_BLOCK_QUANTITY:
			szErrMsg = (char*)"MXIO_V3_ERR_BLOCK_QUANTITY";
			break;
		case MXIO_V3_ERR_TOTAL_BLOCK_SIZE:
			szErrMsg = (char*)"MXIO_V3_ERR_TOTAL_BLOCK_SIZE";
			break;
		case MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION:
			szErrMsg = (char*)"MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION";
			break;
		case MXIO_V3_ERR_PARSE_XML_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_PARSE_XML_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT:
			szErrMsg = (char *)"MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT";
			break;
		case MXIO_V3_PARSING_DEVICE_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_DEVICE_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_CHANNEL_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_CHANNEL_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_INPUT_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_INPUT_NAME_ERROR";
			break;
		default:
			szErrMsg = (char*)"Unknown Error Value";
			break;
		}
		printf("%d Function \"%s\" execution Fail.\nError Message : %s (%d(0x%X))\n",
			   ihConnection, szFunctionName, szErrMsg, iRet, iRet);
	}
	else
	{
		printf("%d Function \"%s\" execution Success\n", ihConnection, szFunctionName);
	}
}
//-----------------------------------------------------------------
typedef struct _FUNC_CHINFO
{
	WORD wIOType;
	WORD wFunc;
	DWORD dwChInfo;
	char *szChInfo;
} FUNC_CHINFO, *pFUNC_CHINFO;
//
FUNC_CHINFO FuncInfo[] =
{
	{IOTYPE_DI,       IOFUNC_DI_CH_MODE,             CHINFO_DI_DI, (char *)"CHINFO_DI_DI"},
	{IOTYPE_DI,       IOFUNC_DI_CH_MODE,             CHINFO_DI_COUNTER, (char *)"CHINFO_DI_COUNTER"},
	{IOTYPE_DO,       IOFUNC_DO_CH_MODE,             CHINFO_DO_DO, (char *)"CHINFO_DO_DO"},
	{IOTYPE_DO,       IOFUNC_DO_CH_MODE,             CHINFO_DO_PULSE, (char *)"CHINFO_DO_PULSE"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_DISABLED, (char *)"CHINFO_AI_DISABLED"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_150mV, (char *)"CHINFO_AI_150mV"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_500mV, (char *)"CHINFO_AI_500mV"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_5V, (char *)"CHINFO_AI_5V"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_10V, (char *)"CHINFO_AI_10V"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_0_20mA, (char *)"CHINFO_AI_0_20mA"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_4_20mA, (char *)"CHINFO_AI_4_20mA"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_0_10V, (char *)"CHINFO_AI_0_10V"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_4_20mA_BurnOut, (char *)"CHINFO_AI_4_20mA_BurnOut"},
	{IOTYPE_AI,          IOFUNC_AI_CH_MODE,          CHINFO_AI_M10V_10V, (char *)"CHINFO_AI_-10V_10V"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_DISABLED, (char *)"CHINFO_TC_DISABLED"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_J_TYPE, (char *)"CHINFO_TC_J_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_K_TYPE, (char *)"CHINFO_TC_K_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_T_TYPE, (char *)"CHINFO_TC_T_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_E_TYPE, (char *)"CHINFO_TC_E_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_R_TYPE, (char *)"CHINFO_TC_R_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_S_TYPE, (char *)"CHINFO_TC_S_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_B_TYPE, (char *)"CHINFO_TC_B_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_N_TYPE, (char *)"CHINFO_TC_N_TYPE"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_78126, (char *)"CHINFO_TC_VOL_78126"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_39062, (char *)"CHINFO_TC_VOL_39062"},
	{IOTYPE_TC,          IOFUNC_TC_CH_MODE,          CHINFO_TC_VOL_19532, (char *)"CHINFO_TC_VOL_19532"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_DISABLED, (char *)"CHINFO_RTD_DISABLED"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT100, (char *)"CHINFO_RTD_PT100"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT200, (char *)"CHINFO_RTD_PT200"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT500, (char *)"CHINFO_RTD_PT500"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT1000, (char *)"CHINFO_RTD_PT1000"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT100, (char *)"CHINFO_RTD_JPT100"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT200, (char *)"CHINFO_RTD_JPT200"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT500, (char *)"CHINFO_RTD_JPT500"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_JPT1000, (char *)"CHINFO_RTD_JPT1000"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI100, (char *)"CHINFO_RTD_NI100"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI200, (char *)"CHINFO_RTD_NI200"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI500, (char *)"CHINFO_RTD_NI500"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI1000, (char *)"CHINFO_RTD_NI1000"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_310, (char *)"CHINFO_RTD_RES_310"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_620, (char *)"CHINFO_RTD_RES_620"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_1250, (char *)"CHINFO_RTD_RES_1250"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_RES_2200, (char *)"CHINFO_RTD_RES_2200"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_PT50, (char *)"CHINFO_RTD_PT50"},
	{IOTYPE_RTD,         IOFUNC_RTD_CH_MODE,         CHINFO_RTD_NI120, (char *)"CHINFO_RTD_NI120"},
};
//-----------------------------------------------------------------
BOOL PrintStrCHInfoStr(int iIndex, CMXIO_V3_4500::MX_RECV_STR_READ_DATA *pImp)
{
	BOOL bIsModeCmd = FALSE;
	WORD wIOType = pImp->MxRecvStrCmd->wIOType;
	WORD wFunc = pImp->MxRecvStrCmd->wFunction;
	BYTE bytSlotNum = pImp->MxRecvStrCmd->bytSlotNum;
	for (int i = 0; i < sizeof(FuncInfo) / sizeof(FuncInfo[0]); i++)
	{
		if (FuncInfo[i].wIOType == wIOType)
		{
			if (FuncInfo[i].wFunc == wFunc)
			{
				bIsModeCmd = TRUE;
				if (FuncInfo[i].dwChInfo == pImp->pRwChannelData[iIndex].unValue.dwValue)
				{
					printf("%s Mode = %d(0x%X): %s\r\n", pImp->MxRecvStrCmd->szDevChName,
						pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue,
						FuncInfo[i].szChInfo);
					return bIsModeCmd;
				}
			}
		}
	}
	if (bIsModeCmd)
	{
		printf("ch[%d] Mode = %d(0x%X) Undefined!! *****\r\n", iIndex,
			pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	}
	return bIsModeCmd;
}
//-----------------------------------------------------------------
void PrtStrErrorString(WORD wErrorCode, char *szName)
{
	if (MXIO_V3_BLOCK_ERR_SIZE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SIZE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_TYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SUBTYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SUBTYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_FUNCTION == wErrorCode)
		printf("[%s] = BLOCK_ERR_FUNCTION\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SLOT == wErrorCode)
		printf("[%s] = BLOCK_ERR_SLOT\n", szName);
	else if (MXIO_V3_BLOCK_ERR_START_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_START_CHANNEL\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TOTAL_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_TOTAL_CHANNEL\n", szName);
	else
		printf("[%s] = Not defined error code= %d\n", szName, wErrorCode);
}
//-----------------------------------------------------------------
void PrtChErrorString(int iIndex, BYTE bytErrorCode)
{
	if (MXIO_V3_CH_OK == bytErrorCode)
		printf("CH[%d] = SUCCESS\n", iIndex);
	else if (MXIO_V3_CH_ERR_LOCKED == bytErrorCode)
		printf("CH[%d] = ERROR_LOCKED\n", iIndex);
	else if (MXIO_V3_CH_ERR_READ_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_READ_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_SUBTYPE == bytErrorCode)
		printf("CH[%d] = ERROR_SUBTYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA_TYPE == bytErrorCode)
		printf("CH[%d] = ERROR_DATA_TYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA == bytErrorCode)
		printf("CH[%d] = ERROR_DATA\n", iIndex);
	else if (MXIO_V3_CH_ERR_DISABLE == bytErrorCode)
		printf("CH[%d] = ERROR_DISABLE\n", iIndex);
	else if (MXIO_V3_CH_ERR_BOOLEAN_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_BOOLEAN_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_WRITE_FAIL== bytErrorCode)
		printf("CH[%d] = MXIO_V3_CH_ERR_WRITE_FAIL\n", iIndex);
	else
		printf("CH[%d] = Not defined error code= %d\n", iIndex, bytErrorCode);
}


//-----------------------------------------------------------------
void PrintReadValue(int iIndex, CMXIO_V3_4500::MX_RECV_READ_BLOCK_DATA *pImp)
{
	printf("iHConnection = %d\n", pImp->iHConnection);
	BYTE bytDataType = pImp->pRwChannelData[iIndex].bytDataType;
	if ((DATA_TYPE_BYTE == bytDataType) || (DATA_TYPE_WORD == bytDataType))
		printf("[%d] = 0x%02X\n", iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue);
	else if ((DATA_TYPE_DWORD == bytDataType) || (DATA_TYPE_INT == bytDataType))
		printf("[%d] = 0x%04X\n", iIndex, pImp->pRwChannelData[iIndex].unValue.dwValue);
	else if (DATA_TYPE_FLOAT == bytDataType)
		printf("[%d] = %.3f\n", iIndex, pImp->pRwChannelData[iIndex].unValue.fValue);
	else if (DATA_TYPE_STRING == bytDataType)
		printf("[%d] = %s\n", iIndex, pImp->pRwChannelData[iIndex].unValue.szValue);
}

//-----------------------------------------------------------------
void *ThreadRead1( void * lParam )
{
	int ihConnection = 0;
    DEVICE_PARAMETERS *io = (DEVICE_PARAMETERS *)lParam;
	sprintf(io->szMACAddr, "%02X-%02X-%02X-%02X-%02X-%02X",
				io->MxMACAddr.bytMAC[0],
				io->MxMACAddr.bytMAC[1],
				io->MxMACAddr.bytMAC[2],
				io->MxMACAddr.bytMAC[3],
				io->MxMACAddr.bytMAC[4],
				io->MxMACAddr.bytMAC[5]);
        printf("IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
		io->IPAddress, io->szMACAddr, io->wPort, io->AccountData->szUserName, io->AccountData->szPwd);
	int iRet = g_pMXIO_V3_4500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(io->MxMACAddr), io->AccountData, &ihConnection);
	CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	printf("IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
	//-----------------------------------------------------
	while(FALSE == io->bStop)
	{
		//Read
	    CMXIO_V3_4500::MX_READ_STR_DATA ReadStrBlockData;
		ReadStrBlockData.wTotalQuantity = 1;
		ReadStrBlockData.pMxStrCmd = new CMXIO_V3_4500::MX_STR_CMD[1];
		ReadStrBlockData.pMxStrCmd[0].wIOType = (WORD)(IOTYPE_IR_WORD);
		ReadStrBlockData.pMxStrCmd[0].wFunction = (WORD)(IOFUNC_IR_CH_VALUE);
	    ReadStrBlockData.pMxStrCmd[0].szDevChName = (char*)"ioThinx_4510@WIR-00";
		//-----------------------
		printf("MXIO_Read(%d)\n", ihConnection);
	    iRet = g_pMXIO_V3_4500->MXIO_Read(ihConnection, &ReadStrBlockData, ReadStrDataCallbackFunc1, NULL);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Read");

        //Free memory
	    delete [] ReadStrBlockData.pMxStrCmd;
		usleep(1000000);
	}

	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Disconnect");
	}
}
//-----------------------------------------------------------------
void *ThreadRead2( void * lParam )
{
	int ihConnection = 0;
    DEVICE_PARAMETERS *io = (DEVICE_PARAMETERS *)lParam;
	sprintf(io->szMACAddr, "%02X-%02X-%02X-%02X-%02X-%02X",
				io->MxMACAddr.bytMAC[0],
				io->MxMACAddr.bytMAC[1],
				io->MxMACAddr.bytMAC[2],
				io->MxMACAddr.bytMAC[3],
				io->MxMACAddr.bytMAC[4],
				io->MxMACAddr.bytMAC[5]);
	printf("IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n",
		io->IPAddress, io->szMACAddr, io->wPort, io->AccountData->szUserName, io->AccountData->szPwd);
	int iRet = g_pMXIO_V3_4500->MXIO_Connect(io->IPAddress, io->wPort, io->dwTimeout, &(io->MxMACAddr), io->AccountData, &ihConnection);
	CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	printf("IP = %s, ihConnection = %d\r\n", io->IPAddress, ihConnection);
	//-----------------------------------------------------
	while(FALSE == io->bStop)
	{
		//Read
	    CMXIO_V3_4500::MX_READ_STR_DATA ReadStrBlockData;
		ReadStrBlockData.wTotalQuantity = 1;
		ReadStrBlockData.pMxStrCmd = new CMXIO_V3_4500::MX_STR_CMD[1];
		ReadStrBlockData.pMxStrCmd[0].wIOType = (WORD)(IOTYPE_IR_WORD);
		ReadStrBlockData.pMxStrCmd[0].wFunction = (WORD)(IOFUNC_IR_CH_VALUE);
	    ReadStrBlockData.pMxStrCmd[0].szDevChName = (char*)"ioThinx_4510@WIR-00";
		//-----------------------
		printf("MXIO_Read(%d)\n", ihConnection);
	    iRet = g_pMXIO_V3_4500->MXIO_Read(ihConnection, &ReadStrBlockData, ReadStrDataCallbackFunc2, NULL);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Read");

        //Free memory
	    delete [] ReadStrBlockData.pMxStrCmd;
		usleep(1000000);
	}

	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Disconnect");
	}
}
//-----------------------------------------------------------------
void CALLBACK ReadStrDataCallbackFunc1(int *pRecvReadBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_STR_READ_DATA *pReadImp = (CMXIO_V3_4500::MX_RECV_STR_READ_DATA *)pRecvReadBlockData;
	if (pReadImp)
	{
		BYTE bytSlotNum = pReadImp->MxRecvStrCmd->bytSlotNum;
		WORD wTotalQuantity = pReadImp->wTotalQuantity;
		if (MXIO_V3_BLOCK_OK == pReadImp->MxRecvStrCmd->wResult) //
		{
			printf("Read by string [%s] Success:\n", pReadImp->MxRecvStrCmd->szDevChName);
			
			for (int i = 0; i < pReadImp->wTotalQuantity; i++)
			{
				if (MXIO_V3_CH_OK == pReadImp->pRwChannelData[i].bytErrorCode)
					PrintStrReadValue(i, pReadImp);
				else
					PrtStrChErrorString(pReadImp->pRwChannelData[i].bytErrorCode, pReadImp->MxRecvStrCmd->szDevChName);
			}
		}
		else
			PrtStrErrorString( pReadImp->MxRecvStrCmd->wResult, pReadImp->MxRecvStrCmd->szDevChName);
	}
}
//-----------------------------------------------------------------
void CALLBACK ReadStrDataCallbackFunc2(int *pRecvReadBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_STR_READ_DATA *pReadImp = (CMXIO_V3_4500::MX_RECV_STR_READ_DATA *)pRecvReadBlockData;
	if (pReadImp)
	{
		BYTE bytSlotNum = pReadImp->MxRecvStrCmd->bytSlotNum;
		WORD wTotalQuantity = pReadImp->wTotalQuantity;
		if (MXIO_V3_BLOCK_OK == pReadImp->MxRecvStrCmd->wResult) //
		{
			printf("Read by string [%s] Success:\n", pReadImp->MxRecvStrCmd->szDevChName);
			
			for (int i = 0; i < pReadImp->wTotalQuantity; i++)
			{
				if (MXIO_V3_CH_OK == pReadImp->pRwChannelData[i].bytErrorCode)
					PrintStrReadValue(i, pReadImp);
				else
					PrtStrChErrorString(pReadImp->pRwChannelData[i].bytErrorCode, pReadImp->MxRecvStrCmd->szDevChName);
			}
		}
		else
			PrtStrErrorString( pReadImp->MxRecvStrCmd->wResult, pReadImp->MxRecvStrCmd->szDevChName);
	}
}
//-----------------------------------------------------------------
void PrintStrReadValue(int iIndex, CMXIO_V3_4500::MX_RECV_STR_READ_DATA *pImp)
{
	if (PrintStrCHInfoStr(iIndex, pImp))
		return;
	BYTE bytDataType = pImp->pRwChannelData[iIndex].bytDataType;
	BYTE bytSlotNum = pImp->MxRecvStrCmd->bytSlotNum;
	char* pszDevChName = pImp->MxRecvStrCmd->szDevChName;
	if ((DATA_TYPE_BYTE == bytDataType) || (DATA_TYPE_WORD == bytDataType))
		printf("%s = %d(0x%02X)\n", pszDevChName, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	else if ((DATA_TYPE_DWORD == bytDataType) || (DATA_TYPE_INT == bytDataType))
		printf("%s = %d(0x%04X)\n", pszDevChName, pImp->pRwChannelData[iIndex].unValue.dwValue, pImp->pRwChannelData[iIndex].unValue.dwValue);
	else if (DATA_TYPE_FLOAT == bytDataType)
		printf("%s = %.3f\n", pszDevChName, pImp->pRwChannelData[iIndex].unValue.fValue);
	else if (DATA_TYPE_STRING == bytDataType)
		printf("%s = %s\n", pszDevChName, pImp->pRwChannelData[iIndex].unValue.szValue);
	else if (DATA_TYPE_DATE == bytDataType)
	{
		printf("%s = %d/%d/%d\n", pszDevChName,
			pImp->pRwChannelData[iIndex].unValue.MxDate.Year,
			pImp->pRwChannelData[iIndex].unValue.MxDate.Month,
			pImp->pRwChannelData[iIndex].unValue.MxDate.Day);
	}
	else if (DATA_TYPE_TIME == bytDataType)
	{
		printf("%s = %d:%d:%d\n", pszDevChName,
			pImp->pRwChannelData[iIndex].unValue.MxTime.Hour,
			pImp->pRwChannelData[iIndex].unValue.MxTime.Min,
			pImp->pRwChannelData[iIndex].unValue.MxTime.Sec);
	}
	else if (DATA_TYPE_UNKNOWN == bytDataType)
		printf("%s = data type return DATA_TYPE_UNKNOWN\n", pszDevChName);
	else
		printf("%s = not defined data type = %d\n", pszDevChName, bytDataType);
}
//-----------------------------------------------------------------
void PrtStrChErrorString(BYTE bytErrorCode, char* pszName)
{
	if (MXIO_V3_CH_OK == bytErrorCode)
		printf("%s = SUCCESS\n", pszName);
	else if (MXIO_V3_CH_ERR_LOCKED == bytErrorCode)
		printf("%s = ERROR_LOCKED\n", pszName);
	else if (MXIO_V3_CH_ERR_READ_ONLY == bytErrorCode)
		printf("%s = ERROR_READ_ONLY\n", pszName);
	else if (MXIO_V3_CH_ERR_SUBTYPE == bytErrorCode)
		printf("%s = ERROR_SUBTYPE\n", pszName);
	else if (MXIO_V3_CH_ERR_DATA_TYPE == bytErrorCode)
		printf("%s = ERROR_DATA_TYPE\n", pszName);
	else if (MXIO_V3_CH_ERR_DATA == bytErrorCode)
		printf("%s = ERROR_DATA\n", pszName);
	else if (MXIO_V3_CH_ERR_DISABLE == bytErrorCode)
		printf("%s = ERROR_DISABLE\n", pszName);
	else if (MXIO_V3_CH_ERR_BOOLEAN_ONLY == bytErrorCode)
		printf("%s = ERROR_BOOLEAN_ONLY\n", pszName);
	else if (MXIO_V3_CH_ERR_WRITE_FAIL == bytErrorCode)
		printf("%s = MXIO_V3_CH_ERR_WRITE_FAIL\n", pszName);
	else if (MXIO_V3_CH_ERR_PARSING_STR_FAIL == bytErrorCode)
		printf("%s = MXIO_V3_CH_ERR_PARSING_STR_FAIL\n", pszName);
	else
		printf("%s = Not defined error code(%d)\n", pszName, bytErrorCode);
}
//-----------------------------------------------------------------

