/*******************************************************************************
 * Copyright Moxa Inc.
 *
 *
 * Date          Author            Comment
 * 02/25/2019    Daniel Chang      Created.
 ******************************************************************************/

 /*!
    \file FloatW.cpp
    \copyright Moxa Inc.
    \brief <b>\a Write \a Float \a Value</b>
    \date 02/25/2019
    \author Daniel Chang
    \version V1.0
    \details
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <MXIO_V3_4500.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
//-----------------------------------------------------------------
#define TRUE    1
#define FALSE   0

#define HMODULE void
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500);
char getch();
void WaitPressAnyKey();
void CheckErr(int ihConnection, int iRet, char *szFunctionName);
void PrtStrErrorString(WORD wErrorCode, char *szName);
void PrtStrChErrorString(BYTE bytErrorCode, char* pszName);

void CALLBACK WriteStrDataCallbackFunc(int *pRecvWriteBlockData, void *pParam);
//-----------------------------------------------------------------
HMODULE *g_hMXIO_V3_4500 = NULL;
CMXIO_V3_4500 *g_pMXIO_V3_4500 = NULL;

//-----------------------------------------------------------------
//! \include FloatW.cpp
int main(int argc, char *argv[])
{
	char bytInputValue[64] = {'\0'};
	char szIP[] = "192.168.127.253";
	CMXIO_V3_4500::MX_MAC_ADDRESS MxMACAddr;
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
    char szMACAddr[18] = {'\0'};//"00-90-e8-00-00-41";
	DWORD dwTimeout = 3000;
	int ihConnection = 0;
	CMXIO_V3_4500::MX_ACCOUNT_DATA AccountData;
	sprintf(AccountData.szUserName,"%s", (char*)"admin");
	sprintf(AccountData.szPwd,"%s", (char*)"moxa");

	//-----------------------------------------------
	char IPAddress[16] = {'\0'};
	char szTempBuf[16] = {'\0'};
	WORD wPort = MXIO_V3_AGENT_PORT;
	sprintf(IPAddress, "%s", szIP);
	if (argc <= 1)
	{
		printf("Cmd Format: [IP Address] [Timeout] [TCP Port]\n");
		strcpy(IPAddress, szIP);
	}
	//
	if (argc > 1)
		sprintf(IPAddress, "%s", argv[1]);
	if (argc > 2)
		dwTimeout = atoi(argv[2]);
	if (argc > 3)
		wPort = atoi(argv[3]);
	printf("Input IP Address=%s, TimeOut=%d, TCP Port=%d\n", IPAddress, dwTimeout, wPort);
	//-----------------------
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500, (char*)"/usr/local/lib/libmxio_4500.so");
#endif
	printf("MXCreateLibrary() g_pMXIO_V3_4500=0x%p return %d(0x%X)\n", g_pMXIO_V3_4500, iRet, iRet);
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Init();
		printf("MXIO_Init() return %d(0x%X)\n", iRet, iRet);
	}
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
		sprintf(szMACAddr, "%02X-%02X-%02X-%02X-%02X-%02X",
				MxMACAddr.bytMAC[0],
				MxMACAddr.bytMAC[1],
				MxMACAddr.bytMAC[2],
				MxMACAddr.bytMAC[3],
				MxMACAddr.bytMAC[4],
				MxMACAddr.bytMAC[5]);
        printf("IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n", 
			IPAddress, szMACAddr, wPort, AccountData.szUserName, AccountData.szPwd);		
        iRet = g_pMXIO_V3_4500->MXIO_Connect(IPAddress, wPort, dwTimeout, &MxMACAddr, &AccountData, &ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	}
	//-----------------------------------------------------
	if(MXIO_V3_OK == iRet)
	{
		//Write
	    CMXIO_V3_4500::MX_WRITE_STR_DATA WriteStrBlockData;
		WriteStrBlockData.pMxStrCmd = new CMXIO_V3_4500::MX_STR_CMD[1];
		WriteStrBlockData.pRwChannelData = new CMXIO_V3_4500::MX_RW_OBJ_DATA[1];
		WriteStrBlockData.pMxStrCmd[0].wIOType = (WORD)(IOTYPE_IR_FLOAT);
		WriteStrBlockData.pMxStrCmd[0].wFunction = (WORD)(IOFUNC_IR_CH_VALUE);
	    WriteStrBlockData.pMxStrCmd[0].szDevChName = (char*)"ioThinx_4510@FIR-00";
		WriteStrBlockData.pRwChannelData[0].bytErrorCode = (BYTE)0;
		WriteStrBlockData.pRwChannelData[0].unValue.fValue = (float)5.123;
	    //-----------------------
	    printf("MXIO_Write(%d)\n", ihConnection);
		iRet = g_pMXIO_V3_4500->MXIO_Write(ihConnection, &WriteStrBlockData, WriteStrDataCallbackFunc, NULL);
		CheckErr(ihConnection, iRet, (char *)"MXIO_Write");

		//Free memory
	    delete [] WriteStrBlockData.pRwChannelData;
        delete [] WriteStrBlockData.pMxStrCmd;
	}

	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Disconnect");
	}
	//-----------------------
	if (NULL != g_hMXIO_V3_4500)
	{
		iRet = g_pMXIO_V3_4500->MXIO_Exit();
		printf("MXIO_Exit() return %d(0x%X)\n", iRet, iRet);
		iRet = MXDestroyLibrary(g_hMXIO_V3_4500, &g_pMXIO_V3_4500);
		printf("MXDestroyLibrary() return %d(0x%X)\n", iRet, iRet);
	}
	WaitPressAnyKey();
	return 0;
}

//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500, char *szLibName)
{
	phMXIO_V3_4500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_4500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_4500 pfnCreateMXIO_V3_4500 = (PFNCREATEMXIO_V3_4500)dlsym(phMXIO_V3_4500, "CreateMXIO_4500");
	if (NULL == pfnCreateMXIO_V3_4500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_4500 = pfnCreateMXIO_V3_4500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_4500, CMXIO_V3_4500 **pMXIO_V3_4500)
{
	PFNDESTROYMXIO_V3_4500 pfnDestroyMXIO_V3_4500 = (PFNDESTROYMXIO_V3_4500)dlsym(hMXIO_V3_4500, "DestroyMXIO_4500");
	if (!pfnDestroyMXIO_V3_4500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_4500(*pMXIO_V3_4500);
	dlclose(hMXIO_V3_4500);
	*pMXIO_V3_4500 = NULL;
	return 0;
}
//-----------------------------------------------------------------
char getch()
{
    int iRet;
	char ch;
	struct termios oldt, newt;

	tcgetattr(STDIN_FILENO, &oldt);
	memcpy(&newt, &oldt, sizeof(newt));
	newt.c_lflag &= ~( ECHO | ICANON | ECHOE | ECHOK |
						ECHONL | ECHOPRT | ECHOKE | ICRNL);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	iRet = read(STDIN_FILENO, &ch, 1);
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if(iRet == 1)
		return ch;
	else
		return 0;
}

void WaitPressAnyKey()
{
	printf("press any key to exit.\n");
	getch();
}

//-----------------------------------------------------------------
//===========================================================================
void CheckErr(int ihConnection, int iRet, char *szFunctionName)
{
	char *szErrMsg = (char*)"MXIO_V3_OK";
	if (iRet != MXIO_V3_OK)
	{
		switch (iRet)
		{
		case MXIO_V3_ILLEGAL_FUNCTION:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_FUNCTION";
			break;
		case MXIO_V3_ILLEGAL_DATA_ADDRESS:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_ADDRESS";
			break;
		case MXIO_V3_ILLEGAL_DATA_VALUE:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_VALUE";
			break;
		case MXIO_V3_SLAVE_DEVICE_FAILURE:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_FAILURE";
			break;
		case MXIO_V3_SLAVE_DEVICE_BUSY:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_BUSY";
			break;
		case MXIO_V3_EIO_TIME_OUT:
			szErrMsg = (char*)"MXIO_V3_EIO_TIME_OUT";
			break;
		case MXIO_V3_EIO_INIT_SOCKETS_FAIL:
			szErrMsg = (char*)"MXIO_V3_EIO_INIT_SOCKETS_FAIL";
			break;
		case MXIO_V3_EIO_CREATING_SOCKET_ERROR:
			szErrMsg = (char*)"MXIO_V3_EIO_CREATING_SOCKET_ERROR";
			break;
		case MXIO_V3_EIO_RESPONSE_BAD:
			szErrMsg = (char*)"MXIO_V3_EIO_RESPONSE_BAD";
			break;
		case MXIO_V3_EIO_SOCKET_DISCONNECT:
			szErrMsg = (char*)"MXIO_V3_EIO_SOCKET_DISCONNECT";
			break;
		case MXIO_V3_PROTOCOL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_PROTOCOL_TYPE_ERROR";
			break;
		case MXIO_V3_PRODUCT_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_PRODUCT_NOT_SUPPORT";
			break;
		case MXIO_V3_HANDLE_ERROR:
		    szErrMsg = (char*)"MXIO_V3_HANDLE_ERROR";
		    break;
		case MXIO_V3_SLOT_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_SLOT_OUT_OF_RANGE";
			break;
		case MXIO_V3_CHANNEL_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_CHANNEL_OUT_OF_RANGE";
			break;
		case MXIO_V3_COIL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_COIL_TYPE_ERROR";
			break;
		case MXIO_V3_REGISTER_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_REGISTER_TYPE_ERROR";
			break;
		case MXIO_V3_FUNCTION_NOT_SUPPORT:
		    szErrMsg = (char*)"MXIO_V3_FUNCTION_NOT_SUPPORT";
		    break;
		case MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_INPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_INPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_EIO_PASSWORD_INCORRECT:
			szErrMsg = (char*)"MXIO_V3_EIO_PASSWORD_INCORRECT";
			break;
		case MXIO_V3_SLOT_NOT_EXIST:
			szErrMsg = (char*)"MXIO_V3_SLOT_NOT_EXIST";
			break;
		case MXIO_V3_FIRMWARE_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_FIRMWARE_NOT_SUPPORT";
			break;
		case MXIO_V3_CREATE_MUTEX_FAIL:
			szErrMsg = (char*)"MXIO_V3_CREATE_MUTEX_FAIL";
			break;
		case MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_FUNCTION_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_ERR_FUNCTION_NOT_SUPPORT";
			break;
		case MXIO_V3_ERR_INVALID_HANDLE:
			szErrMsg = (char*)"MXIO_V3_ERR_INVALID_HANDLE";
			break;
		case MXIO_V3_ERR_BLOCK_QUANTITY:
			szErrMsg = (char*)"MXIO_V3_ERR_BLOCK_QUANTITY";
			break;
		case MXIO_V3_ERR_TOTAL_BLOCK_SIZE:
			szErrMsg = (char*)"MXIO_V3_ERR_TOTAL_BLOCK_SIZE";
			break;
		case MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION:
			szErrMsg = (char*)"MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION";
			break;
		case MXIO_V3_ERR_PARSE_XML_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_PARSE_XML_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ACCOUNT_VERIFY_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT:
			szErrMsg = (char *)"MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT";
			break;
		case MXIO_V3_PARSING_DEVICE_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_DEVICE_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_CHANNEL_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_CHANNEL_NAME_ERROR";
			break;
		case MXIO_V3_PARSING_INPUT_NAME_ERROR:
			szErrMsg = (char *)"MXIO_V3_PARSING_INPUT_NAME_ERROR";
			break;
		default:
			szErrMsg = (char*)"Unknown Error Value";
			break;
		}
		printf("%d Function \"%s\" execution Fail.\nError Message : %s (%d(0x%X))\n",
			   ihConnection, szFunctionName, szErrMsg, iRet, iRet);
	}
	else
	{
		printf("%d Function \"%s\" execution Success\n", ihConnection, szFunctionName);
	}
}

//-----------------------------------------------------------------
void PrtChErrorString(int iIndex, BYTE bytErrorCode)
{
	if (MXIO_V3_CH_OK == bytErrorCode)
		printf("CH[%d] = SUCCESS\n", iIndex);
	else if (MXIO_V3_CH_ERR_LOCKED == bytErrorCode)
		printf("CH[%d] = ERROR_LOCKED\n", iIndex);
	else if (MXIO_V3_CH_ERR_READ_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_READ_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_SUBTYPE == bytErrorCode)
		printf("CH[%d] = ERROR_SUBTYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA_TYPE == bytErrorCode)
		printf("CH[%d] = ERROR_DATA_TYPE\n", iIndex);
	else if (MXIO_V3_CH_ERR_DATA == bytErrorCode)
		printf("CH[%d] = ERROR_DATA\n", iIndex);
	else if (MXIO_V3_CH_ERR_DISABLE == bytErrorCode)
		printf("CH[%d] = ERROR_DISABLE\n", iIndex);
	else if (MXIO_V3_CH_ERR_BOOLEAN_ONLY == bytErrorCode)
		printf("CH[%d] = ERROR_BOOLEAN_ONLY\n", iIndex);
	else if (MXIO_V3_CH_ERR_WRITE_FAIL== bytErrorCode)
		printf("CH[%d] = MXIO_V3_CH_ERR_WRITE_FAIL\n", iIndex);
	else
		printf("CH[%d] = Not defined error code= %d\n", iIndex, bytErrorCode);
}

//-----------------------------------------------------------------
void CALLBACK WriteStrDataCallbackFunc(int *pRecvWriteBlockData, void *pParam)
{
	CMXIO_V3_4500::MX_RECV_STR_WRITE_DATA *pWriteImp = (CMXIO_V3_4500::MX_RECV_STR_WRITE_DATA *)pRecvWriteBlockData;
	if (pWriteImp)
	{
		BYTE bytSlotNum = pWriteImp->MxRecvStrCmd->bytSlotNum;
		WORD wTotalQuantity = pWriteImp->wTotalQuantity;
		if (MXIO_V3_BLOCK_OK == pWriteImp->MxRecvStrCmd->wResult) //
		{
			printf("Write by string [%s] Success:\n", pWriteImp->MxRecvStrCmd->szDevChName);
			
			for (int i = 0; i < pWriteImp->wTotalQuantity; i++)
			{
				PrtStrChErrorString(pWriteImp->pRwChannelData[i].bytErrorCode, pWriteImp->MxRecvStrCmd->szDevChName);
			}
		}
		else
			PrtStrErrorString( pWriteImp->MxRecvStrCmd->wResult, pWriteImp->MxRecvStrCmd->szDevChName);
	}
}
//-----------------------------------------------------------------
void PrtStrErrorString(WORD wErrorCode, char *szName)
{
	if (MXIO_V3_BLOCK_ERR_SIZE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SIZE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_TYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SUBTYPE == wErrorCode)
		printf("[%s] = BLOCK_ERR_SUBTYPE\n", szName);
	else if (MXIO_V3_BLOCK_ERR_FUNCTION == wErrorCode)
		printf("[%s] = BLOCK_ERR_FUNCTION\n", szName);
	else if (MXIO_V3_BLOCK_ERR_SLOT == wErrorCode)
		printf("[%s] = BLOCK_ERR_SLOT\n", szName);
	else if (MXIO_V3_BLOCK_ERR_START_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_START_CHANNEL\n", szName);
	else if (MXIO_V3_BLOCK_ERR_TOTAL_OBJ == wErrorCode)
		printf("[%s] = BLOCK_ERR_TOTAL_CHANNEL\n", szName);
	else
		printf("[%s] = Not defined error code= %d\n", szName, wErrorCode);
}
//-----------------------------------------------------------------
void PrtStrChErrorString(BYTE bytErrorCode, char* pszName)
{
	if (MXIO_V3_CH_OK == bytErrorCode)
		printf("%s = SUCCESS\n", pszName);
	else if (MXIO_V3_CH_ERR_LOCKED == bytErrorCode)
		printf("%s = ERROR_LOCKED\n", pszName);
	else if (MXIO_V3_CH_ERR_READ_ONLY == bytErrorCode)
		printf("%s = ERROR_READ_ONLY\n", pszName);
	else if (MXIO_V3_CH_ERR_SUBTYPE == bytErrorCode)
		printf("%s = ERROR_SUBTYPE\n", pszName);
	else if (MXIO_V3_CH_ERR_DATA_TYPE == bytErrorCode)
		printf("%s = ERROR_DATA_TYPE\n", pszName);
	else if (MXIO_V3_CH_ERR_DATA == bytErrorCode)
		printf("%s = ERROR_DATA\n", pszName);
	else if (MXIO_V3_CH_ERR_DISABLE == bytErrorCode)
		printf("%s = ERROR_DISABLE\n", pszName);
	else if (MXIO_V3_CH_ERR_BOOLEAN_ONLY == bytErrorCode)
		printf("%s = ERROR_BOOLEAN_ONLY\n", pszName);
	else if (MXIO_V3_CH_ERR_WRITE_FAIL == bytErrorCode)
		printf("%s = MXIO_V3_CH_ERR_WRITE_FAIL\n", pszName);
	else if (MXIO_V3_CH_ERR_PARSING_STR_FAIL == bytErrorCode)
		printf("%s = MXIO_V3_CH_ERR_PARSING_STR_FAIL\n", pszName);
	else
		printf("%s = Not defined error code(%d)\n", pszName, bytErrorCode);
}
//-----------------------------------------------------------------
